"""A color definition used in the document.

Maps a color to equivalents in all available color models.

- Used in `Application.backgroundColor` and `foregroundColor` properties, in
`Channel.color`, in `ColorSampler.color`, and in `TextItem.color`
- Passed to `PathItem.fillPath()`, `Selection.fill()`, and
`Selection.stroke()`.

"""

# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api.colors.cmyk import CMYKColor
from photoshop.api.colors.gray import GrayColor
from photoshop.api.colors.hsb import HSBColor
from photoshop.api.colors.lab import LabColor
from photoshop.api.colors.rgb import RGBColor
from photoshop.api.enumerations import ColorModel


class SolidColor(Photoshop):
    object_name = "SolidColor"

    def __init__(self, parent=None):
        super().__init__(parent=parent)

    @property
    def cmyk(self) -> CMYKColor:
        """The CMYK color mode.

        Returns:
            .colors.cmyk.CMYKColor:

        """
        return CMYKColor(self.app.cmyk)

    @cmyk.setter
    def cmyk(self, value: CMYKColor):
        self.app.cmyk = value

    @property
    def gray(self) -> GrayColor:
        return GrayColor(self.app.gray)

    @property
    def hsb(self) -> HSBColor:
        return HSBColor(self.app.hsb)

    @hsb.setter
    def hsb(self, value: HSBColor):
        self.app.hsb = value

    @property
    def lab(self) -> LabColor:
        return LabColor(self.app.lab)

    @lab.setter
    def lab(self, value: LabColor):
        self.app.lab = value

    @property
    def model(self) -> ColorModel:
        """The color model."""
        return ColorModel(self.app.model)

    @model.setter
    def model(self, value: ColorModel):
        """The color model."""
        self.app.model = value

    @property
    def nearestWebColor(self) -> RGBColor:
        """The nearest web color to the current color."""
        return RGBColor(self.app.NearestWebColor)

    @property
    def rgb(self) -> RGBColor:
        """The RGB color mode."""
        return RGBColor(self.app.rgb)

    @rgb.setter
    def rgb(self, value: RGBColor):
        self.app.rgb = value

    def isEqual(self, color: RGBColor):
        """`SolidColor` object is visually equal to the specified color."""
        return self.app.isEqual(color)



# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api.enumerations import AntiAlias
from photoshop.api.enumerations import AutoKernType
from photoshop.api.enumerations import Direction
from photoshop.api.enumerations import Justification
from photoshop.api.enumerations import Language
from photoshop.api.enumerations import StrikeThruType
from photoshop.api.enumerations import TextComposer
from photoshop.api.enumerations import TextType
from photoshop.api.solid_color import SolidColor
from photoshop.api.text_font import TextFont


class TextItem(Photoshop):
    object_name = "Application"

    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def alternateLigatures(self):
        return self.app.alternateLigatures

    @alternateLigatures.setter
    def alternateLigatures(self, value):
        self.app.alternateLigatures = value

    @property
    def antiAliasMethod(self) -> AntiAlias:
        """The method of anti aliasing to use."""
        return AntiAlias(self.app.antiAliasMethod)

    @antiAliasMethod.setter
    def antiAliasMethod(self, value):
        self.app.antiAliasMethod = value

    @property
    def autoKerning(self) -> AutoKernType:
        """The auto kerning option to use."""
        return AutoKernType(self.app.autoKerning)

    @autoKerning.setter
    def autoKerning(self, value):
        self.app.autoKerning = value

    @property
    def autoLeadingAmount(self):
        return self.app.autoLeadingAmount

    @autoLeadingAmount.setter
    def autoLeadingAmount(self, value):
        """The percentage to use for auto (default) leading (in points).

        Valid only when useAutoLeading = True.

        """
        self.app.useAutoLeading = True
        self.app.autoLeadingAmount = value

    @property
    def baselineShift(self):
        """The unit value to use in the baseline offset of text."""
        return self.app.baselineShift

    @baselineShift.setter
    def baselineShift(self, value):
        self.app.baselineShift = value

    @property
    def capitalization(self):
        """Gets text case."""
        return self.app.capitalization

    @capitalization.setter
    def capitalization(self, value):
        """Sets text case."""
        self.app.capitalization = value

    @property
    def color(self) -> SolidColor:
        """The text color."""
        return SolidColor(self.app.color)

    @color.setter
    def color(self, color_value):
        """The color of textItem."""
        self.app.color = color_value

    @property
    def contents(self) -> str:
        """The actual text in the layer."""
        return self.app.contents

    @contents.setter
    def contents(self, text: str):
        """Set the actual text in the layer.

        Args:
            text: The actual text.

        """
        self.app.contents = text

    @property
    def desiredGlyphScaling(self):
        """The desired amount by which to scale the horizontal size of the
        text letters. A percentage value; at 100, the width of characters is
        not scaled."""
        return self.app.desiredGlyphScaling

    @desiredGlyphScaling.setter
    def desiredGlyphScaling(self, value):
        self.app.desiredGlyphScaling = value

    @property
    def desiredLetterScaling(self):
        """The amount of space between letters .
        (at 0, no space is added between letters).
        Equivalent to Letter Spacing in the Justification
        dialog (Select Justification on the Paragraphs palette menu).
        Valid only when justification = Justification.CENTERJUSTIFIED,
                                                      FULLYJUSTIFIED,
                                                      LEFTJUSTIFIED, or
                                        Justification.RIGHTJUSTIFIED.
        When used, the minimumLetterScaling and
        maximumLetterScaling values are also required.

        """
        return self.app.desiredLetterScaling

    @desiredLetterScaling.setter
    def desiredLetterScaling(self, value):
        self.app.desiredGlyphScaling = value

    @property
    def desiredWordScaling(self):
        """
        The amount (percentage) of space
        between words (at 100, no additional space is added
        between words).
        Equivalent to Word Spacing in the Justification
        dialog (Select Justification on the Paragraphs
        palette menu).
        Valid only when justification =
        Justification.CENTERJUSTIFIED,
        FULLYJUSTIFIED, LEFTJUSTIFIED, or
        Justification.RIGHTJUSTIFIED.
        When used, the minimumWordScaling and
        maximumWordScaling values are also required

        """
        return self.app.desiredWordScaling

    @desiredWordScaling.setter
    def desiredWordScaling(self, value):
        self.app.desiredWordScaling = value

    @property
    def direction(self):
        """The text orientation."""
        return Direction(self.app.direction)

    @direction.setter
    def direction(self, value):
        self.app.direction = value

    @property
    def fauxBold(self):
        """True to use faux bold (default: false).

        Setting this to true is equivalent to selecting text and
        clicking Faux Bold in the Character palette.

        """
        return self.app.fauxBold

    @fauxBold.setter
    def fauxBold(self, value):
        self.app.fauxBold = value

    @property
    def fauxItalic(self):
        """True to use faux italic (default: false).

        Setting this to true is equivalent to selecting text and
        clicking Faux Italic in the Character palette.

        """
        return self.app.fauxItalic

    @fauxItalic.setter
    def fauxItalic(self, value):
        self.app.fauxItalic = value

    @property
    def firstLineIndent(self):
        """The amount (unit value) to indent the first line of paragraphs."""
        return self.app.firstLineIndent

    @firstLineIndent.setter
    def firstLineIndent(self, value):
        self.app.firstLineIndent = value

    @property
    def font(self):
        """.text_font.TextFont: Current font."""
        return TextFont(self.app.font)

    @font.setter
    def font(self, text_font):
        self.app.font = text_font

    @property
    def hangingPunctuation(self):
        """True to use Roman hanging punctuation."""
        return self.app.hangingPunctuation

    @hangingPunctuation.setter
    def hangingPunctuation(self, value):
        self.app.hangingPunctuation = value

    @property
    def height(self):
        """int:The height of the bounding box for paragraph text."""
        return self.app.height

    @height.setter
    def height(self, value):
        self.app.height = value

    @property
    def horizontalScale(self):
        """Character scaling (horizontal) in proportion to verticalScale

        (a percentage value).

        """
        return self.app.horizontalScale

    @horizontalScale.setter
    def horizontalScale(self, value):
        self.app.horizontalScale = value

    @property
    def hyphenateAfterFirst(self):
        """The number of letters after which hyphenation in word wrap is allowed."""
        return self.app.hyphenateAfterFirst

    @hyphenateAfterFirst.setter
    def hyphenateAfterFirst(self, value):
        self.app.hyphenateAfterFirst = value

    @property
    def hyphenateBeforeLast(self):
        """The number of letters before which hyphenation in word wrap is allowed."""
        return self.app.hyphenateBeforeLast

    @hyphenateBeforeLast.setter
    def hyphenateBeforeLast(self, value):
        self.app.hyphenateBeforeLast = value

    @property
    def hyphenateCapitalWords(self):
        """True to allow hyphenation in word wrap of capitalized words"""
        return self.app.hyphenateCapitalWords

    @hyphenateCapitalWords.setter
    def hyphenateCapitalWords(self, value):
        self.app.hyphenateCapitalWords = value

    @property
    def hyphenateWordsLongerThan(self):
        """The minimum number of letters a word must have in order for
        hyphenation in word wrap to be allowed."""
        return self.app.hyphenateWordsLongerThan

    @hyphenateWordsLongerThan.setter
    def hyphenateWordsLongerThan(self, value):
        self.app.hyphenateWordsLongerThan = value

    @property
    def hyphenation(self):
        """True to use hyphenation in word wrap."""
        return self.app.hyphenation

    @hyphenation.setter
    def hyphenation(self, value):
        self.app.hyphenation = value

    @property
    def hyphenationZone(self):
        """The distance at the end of a line that will cause a word to break in
        unjustified type."""
        return self.app.hyphenationZone

    @hyphenationZone.setter
    def hyphenationZone(self, value):
        self.app.hyphenationZone = value

    @property
    def hyphenLimit(self):
        return self.app.hyphenLimit

    @hyphenLimit.setter
    def hyphenLimit(self, value):
        self.app.hyphenLimit = value

    @property
    def justification(self):
        """The paragraph justification."""
        return Justification(self.app.justification)

    @justification.setter
    def justification(self, value):
        self.app.justification = value

    @property
    def kind(self):
        return TextType(self.app.kind)

    @kind.setter
    def kind(self, kind_type):
        self.app.kind = kind_type

    @property
    def language(self):
        return Language(self.app.language)

    @language.setter
    def language(self, text):
        self.app.language = text

    @property
    def leading(self):
        return self.app.leading

    @leading.setter
    def leading(self, value):
        self.app.leading = value

    @property
    def leftIndent(self):
        """The amoun of space to indent text from the left."""
        return self.app.leftIndent

    @leftIndent.setter
    def leftIndent(self, value):
        self.app.leftIndent = value

    @property
    def ligatures(self):
        """True to use ligatures."""
        return self.app.ligatures

    @ligatures.setter
    def ligatures(self, value):
        self.app.ligatures = value

    @property
    def maximumGlyphScaling(self):
        """The maximum amount to scale the horizontal size of the text letters
        (a percentage value; at 100, the width of characters is not scaled).

        Valid only when justification =
        Justification.CENTERJUSTIFIED,
        FULLYJUSTIFIED, LEFTJUSTIFIED, or
        Justification.RIGHTJUSTIFIED.
        When used, the minimumGlyphScaling and
        desiredGlyphScaling values are also required.

        """
        return self.app.maximumGlyphScaling

    @maximumGlyphScaling.setter
    def maximumGlyphScaling(self, value):
        self.app.maximumGlyphScaling = value

    @property
    def maximumLetterScaling(self):
        """The maximum amount of space to allow between letters

        (at 0, no space is added between letters).
        Equivalent to Letter Spacing in the Justification
        dialog (Select Justification on the Paragraphs
        palette menu).
        Valid only when justification =
        Justification.CENTERJUSTIFIED,
        FULLYJUSTIFIED, LEFTJUSTIFIED, or
        Justification.RIGHTJUSTIFIED.
        When used, the minimumLetterScaling and
        desiredLetterScaling values are also required.

        """
        return self.app.maximumLetterScaling

    @maximumLetterScaling.setter
    def maximumLetterScaling(self, value):
        self.app.maximumLetterScaling = value

    @property
    def maximumWordScaling(self):
        return self.app.maximumWordScaling

    @maximumWordScaling.setter
    def maximumWordScaling(self, value):
        self.app.maximumWordScaling = value

    @property
    def minimumGlyphScaling(self):
        """The minimum amount to scale the horizontal size of the text letters
        (a percentage value; at 100, the width of characters is not scaled).

        Valid only when justification =
        Justification.CENTERJUSTIFIED,
        FULLYJUSTIFIED, LEFTJUSTIFIED, or
        Justification.RIGHTJUSTIFIED.
        When used, the maximumGlyphScaling and
        desiredGlyphScaling values are also required.

        """
        return self.app.minimumGlyphScaling

    @minimumGlyphScaling.setter
    def minimumGlyphScaling(self, value):
        self.app.minimumGlyphScaling = value

    @property
    def minimumLetterScaling(self):
        """The minimum amount of space to allow between letters

        (a percentage value; at 0, no space is removed between letters).

        Equivalent to Letter Spacing in the Justification
        dialog (Select Justification on the Paragraphs
        palette menu).
        Valid only when justification =
        Justification.CENTERJUSTIFIED,
        FULLYJUSTIFIED, LEFTJUSTIFIED, or
        Justification.RIGHTJUSTIFIED.
        When used, the maximumLetterScaling and
        desiredLetterScaling values are also required.

        """
        return self.app.minimumLetterScaling

    @minimumLetterScaling.setter
    def minimumLetterScaling(self, value):
        self.app.minimumLetterScaling = value

    @property
    def minimumWordScaling(self):
        """The minimum amount of space to allow between words

        (a percentage value; at 100, no additional space is removed between words).

        Equivalent to Word Spacing in the Justification
        dialog (Select Justification on the Paragraphs
        palette menu).
        Valid only when justification =
        Justification.CENTERJUSTIFIED,
        FULLYJUSTIFIED, LEFTJUSTIFIED, or
        Justification.RIGHTJUSTIFIED.
        When used, the maximumWordScaling and
        desiredWordScaling values are also required.

        """
        return self.app.minimumWordScaling

    @minimumWordScaling.setter
    def minimumWordScaling(self, value):
        self.app.minimumWordScaling = value

    @property
    def noBreak(self):
        """True to disallow line breaks in this text.

        Tip: When true for many consecutive characters, can
        prevent word wrap and thus may prevent some
        text from appearing on the screen.

        """
        return self.app.noBreak

    @noBreak.setter
    def noBreak(self, value):
        self.app.noBreak = value

    @property
    def oldStyle(self):
        return self.app.oldStyle

    @oldStyle.setter
    def oldStyle(self, value):
        self.app.oldStyle = value

    @property
    def parent(self):
        return self.app.parent

    @parent.setter
    def parent(self, value):
        self.app.parent = value

    @property
    def position(self):
        return self.app.position

    @position.setter
    def position(self, array):
        """The position of the origin for the text.

        The array must contain two values. Setting this property is basically
        equivalent to clicking the text tool at a point in the documents to
        create the point of origin for text.

        """
        self.app.position = array

    @property
    def rightIndent(self):
        return self.app.rightIndent

    @rightIndent.setter
    def rightIndent(self, value):
        self.app.rightIndent = value

    @property
    def size(self):
        """The font size in UnitValue.

        NOTE: Type was points for CS3 and older.

        """
        return self.app.size

    @size.setter
    def size(self, value):
        self.app.size = value

    @property
    def spaceAfter(self):
        """The amount of space to use after each paragraph."""
        return self.app.spaceAfter

    @spaceAfter.setter
    def spaceAfter(self, value):
        self.app.spaceAfter = value

    @property
    def spaceBefore(self):
        return self.app.spaceBefore

    @spaceBefore.setter
    def spaceBefore(self, value):
        self.app.spaceBefore = value

    @property
    def strikeThru(self):
        """The text strike-through option to use."""
        return StrikeThruType(self.app.strikeThru)

    @strikeThru.setter
    def strikeThru(self, value):
        self.app.strikeThru = value

    @property
    def textComposer(self):
        return TextComposer(self.app.textComposer)

    @textComposer.setter
    def textComposer(self, value):
        self.app.textComposer = value

    @property
    def tracking(self):
        return self.app.tracking

    @tracking.setter
    def tracking(self, value):
        self.app.tracking = value

    @property
    def underline(self):
        """The text underlining options."""
        return self.app.underline

    @underline.setter
    def underline(self, value):
        self.app.underline = value

    @property
    def useAutoLeading(self):
        return self.app.useAutoLeading

    @useAutoLeading.setter
    def useAutoLeading(self, value):
        self.app.useAutoLeading = value

    @property
    def verticalScale(self):
        return self.app.verticalScale

    @verticalScale.setter
    def verticalScale(self, value):
        self.app.verticalScale = value

    @property
    def warpBend(self):
        """The warp bend percentage."""
        return self.app.warpBend

    @warpBend.setter
    def warpBend(self, value):
        self.app.warpBend = value

    @property
    def warpDirection(self) -> Direction:
        """The warp direction."""
        return Direction(self.app.warpDirection)

    @warpDirection.setter
    def warpDirection(self, value):
        self.app.warpDirection = value

    @property
    def warpHorizontalDistortion(self):
        return self.app.warpHorizontalDistortion

    @warpHorizontalDistortion.setter
    def warpHorizontalDistortion(self, value):
        self.app.warpHorizontalDistortion = value

    @property
    def warpStyle(self):
        """The style of warp to use."""
        return self.app.warpStyle

    @warpStyle.setter
    def warpStyle(self, value):
        self.app.warpStyle = value

    @property
    def warpVerticalDistortion(self):
        return self.app.warpVerticalDistortion

    @warpVerticalDistortion.setter
    def warpVerticalDistortion(self, value):
        self.app.warpVerticalDistortion = value

    @property
    def width(self):
        """The width of the bounding box for

        paragraph text.
        Valid only when kind = TextType.PARAGRAPHTEXT.

        """
        return self.app.width

    @width.setter
    def width(self, value: float):
        """The width of the bounding box for

        paragraph text.
        Valid only when kind = TextType.PARAGRAPHTEXT.

        """
        self.app.width = value

    def convertToShape(self):
        """Converts the text item and its containing layer to a fill layer with the
        text changed to a clipping path."""
        return self.app.convertToShape()

    def createPath(self):
        """Creates a clipping path from the outlines of the actual text items

        (such as letters or words).

        """
        return self.app.createPath()



# Import local modules
from photoshop.api._core import Photoshop


class EPSOpenOptions(Photoshop):
    """Options for saving a document in EPS format.

    using the `Document.saveAs()`

    """

    object_name = "EPSOpenOptions"

    def __init__(self):
        super().__init__()

    @property
    def antiAlias(self):
        return self.app.antiAlias

    @property
    def constrainProportions(self):
        return self.app.constrainProportions

    @property
    def height(self):
        return self.app.height

    @property
    def mode(self):
        return self.app.mode

    @property
    def resolution(self):
        return self.app.resolution

    @property
    def width(self):
        return self.app.width

    @property
    def embedColorProfile(self):
        return self.app.embedColorProfile

    @embedColorProfile.setter
    def embedColorProfile(self, boolean):
        self.app.embedColorProfile = boolean



# Import local modules
from photoshop.api.colors.cmyk import CMYKColor
from photoshop.api.colors.gray import GrayColor
from photoshop.api.colors.hsb import HSBColor
from photoshop.api.colors.lab import LabColor
from photoshop.api.colors.rgb import RGBColor


__all__ = [
    "CMYKColor",
    "HSBColor",
    "LabColor",
    "RGBColor",
    "GrayColor",
]



# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api._document import Document
from photoshop.api.enumerations import BitsPerChannelType
from photoshop.api.enumerations import DocumentFill
from photoshop.api.enumerations import NewDocumentMode
from photoshop.api.errors import PhotoshopPythonAPIError


# pylint: disable=too-many-public-methods, too-many-arguments
class Documents(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    def __len__(self) -> int:
        return self.length

    def add(
        self,
        width: int = 960,
        height: int = 540,
        resolution: float = 72.0,
        name: str = None,
        mode: int = NewDocumentMode.NewRGB,
        initialFill: int = DocumentFill.White,
        pixelAspectRatio: float = 1.0,
        bitsPerChannel: int = BitsPerChannelType.Document8Bits,
        colorProfileName: str = None,
    ) -> Document:
        """Creates a new document object and adds it to this collections.

        Args:
            width (int): The width of the document.
            height (int): The height of the document.
            resolution (int): The resolution of the document (in pixels per inch)
            name (str): The name of the document.
            mode (): The document mode.
            initialFill : The initial fill of the document.
            pixelAspectRatio: The initial pixel aspect ratio of the document.
                                Default is `1.0`, the range is `0.1-10.00`.
            bitsPerChannel: The number of bits per channel.
            colorProfileName: The name of color profile for document.

        Returns:
            .Document: Document instance.

        """
        return Document(
            self.app.add(
                width,
                height,
                resolution,
                name,
                mode,
                initialFill,
                pixelAspectRatio,
                bitsPerChannel,
                colorProfileName,
            )
        )

    def __iter__(self) -> Document:
        for doc in self.app:
            yield Document(doc)

    def __getitem__(self, item) -> Document:
        try:
            return Document(self.app[item])
        except IndexError:
            raise PhotoshopPythonAPIError("Currently Photoshop did not find Documents.")

    @property
    def length(self) -> int:
        return len(list(self.app))

    def getByName(self, document_name: str):
        return Document(self.app.getByName(document_name))



# Import built-in modules
from pathlib import Path

# Import local modules
from photoshop.api._core import Photoshop


class Preferences(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def additionalPluginFolder(self):
        """The path to an additional plug-in folder."""
        return Path(self.app.additionalPluginFolder)

    @property
    def appendExtension(self):
        return self.app.appendExtension

    @property
    def askBeforeSavingLayeredTIFF(self) -> bool:
        return self.app.askBeforeSavingLayeredTIFF

    @property
    def autoUpdateOpenDocuments(self) -> bool:
        """True to automatically update open documents."""
        return self.app.autoUpdateOpenDocuments

    @autoUpdateOpenDocuments.setter
    def autoUpdateOpenDocuments(self, boolean: bool):
        """True to automatically update open documents."""
        self.app.autoUpdateOpenDocuments = boolean

    @property
    def beepWhenDone(self) -> bool:
        """True to beep when a process."""
        return self.app.beepWhenDone

    @beepWhenDone.setter
    def beepWhenDone(self, boolean):
        self.app.beepWhenDone = boolean

    @property
    def colorChannelsInColor(self):
        """True to display component channels in the Channels palette in
        color."""
        return self.app.colorChannelsInColor

    @colorChannelsInColor.setter
    def colorChannelsInColor(self, value):
        self.app.colorChannelsInColor = value

    @property
    def colorPicker(self):
        """The preferred color selection tool."""
        return self.app.colorPicker

    @colorPicker.setter
    def colorPicker(self, value):
        self.app.colorPicker = value

    @property
    def columnGutter(self):
        return self.app.columnGutter

    @columnGutter.setter
    def columnGutter(self, value):
        self.app.columnGutter = value

    @property
    def columnWidth(self):
        return self.app.columnWidth

    @columnWidth.setter
    def columnWidth(self, value):
        self.app.columnWidth = value

    @property
    def createFirstSnapshot(self):
        """Automatically make the first snapshot when a new document is
        created."""
        return self.app.createFirstSnapshot

    @createFirstSnapshot.setter
    def createFirstSnapshot(self, boolean):
        self.app.createFirstSnapshot = boolean

    @property
    def dynamicColorSliders(self):
        return self.app.dynamicColorSliders

    @dynamicColorSliders.setter
    def dynamicColorSliders(self, boolean):
        self.app.dynamicColorSliders = boolean

    @property
    def editLogItems(self) -> bool:
        """The preferred level of detail in the history log."""
        return self.app.editLogItems

    @editLogItems.setter
    def editLogItems(self, boolean: bool):
        """The preferred level of detail in the history log.

        Valid only when useHistoryLog = True

        """
        self.app.editLogItems = boolean

    @property
    def exportClipboard(self):
        """Retain Photoshop contents on the clipboard after exit the app."""
        return self.app.exportClipboard

    @exportClipboard.setter
    def exportClipboard(self, boolean: bool):
        self.app.exportClipboard = boolean

    @property
    def fontPreviewSize(self):
        return self.app.fontPreviewSize

    @fontPreviewSize.setter
    def fontPreviewSize(self, value):
        self.app.fontPreviewSize = value

    @property
    def fullSizePreview(self):
        return self.app.fullSizePreview

    @fullSizePreview.setter
    def fullSizePreview(self, value):
        self.app.fullSizePreview = value

    @property
    def gamutWarningOpacity(self):
        return self.app.gamutWarningOpacity

    @property
    def rulerUnits(self):
        return self.app.rulerUnits

    @rulerUnits.setter
    def rulerUnits(self, value):
        self.app.rulerUnits = value



"""This class provides all photoshop API core functions."""

# Import built-in modules
import os
import platform
from typing import Any
from typing import List
import winreg

# Import third-party modules
from comtypes.client import CreateObject

# Import local modules
from photoshop.api import constants
from photoshop.api.errors import PhotoshopPythonAPIError


class Photoshop(object):
    _root = "Photoshop"
    REG_PATH = "SOFTWARE\\Adobe\\Photoshop"
    _object_name = "Application"
    object_name: str = None

    def __init__(self, ps_version: str = None, parent: Any = None):
        self._program_name = None
        self._has_parent = False
        version_mappings = constants.PHOTOSHOP_VERSION_MAPPINGS
        self.photoshop_version = os.getenv("PS_VERSION", ps_version)
        if self.photoshop_version:
            # Store current photoshop version.
            os.environ["PS_VERSION"] = self.photoshop_version
        self.app_id = version_mappings.get(self.photoshop_version, "0")
        try:
            self.app = self.instance_app(self.app_id)
        except OSError:
            # get photoshop from registration.
            try:
                self.app = self.instance_app(self._get_program_id())
            except OSError:
                raise PhotoshopPythonAPIError(
                    "Please check if you have Photoshop installed correctly.",
                )
        if parent:
            self.adobe = self.app
            self.app = parent
            self._has_parent = True

    @property
    def typename(self):
        return self.__class__.__name__

    def __call__(self, *args, **kwargs):
        return self.app

    def __str__(self):
        return f"{self.__class__.__name__} <{self._program_name}>"

    def __repr__(self):
        return self

    def __getattribute__(self, item):
        try:
            return super().__getattribute__(item)
        except AttributeError:
            return getattr(self.app, item)

    @staticmethod
    def open_key(key):
        """Open the register key.

        Args:
            key (str): The key of register.

        Returns:
            str: The handle to the specified key.

        """
        machine_type = platform.machine()
        mappings = {"AMD64": winreg.KEY_WOW64_64KEY}
        platform_type = mappings.get(machine_type, winreg.KEY_WOW64_32KEY)
        try:
            return winreg.OpenKey(
                winreg.HKEY_LOCAL_MACHINE,
                key,
                access=winreg.KEY_READ | platform_type,
            )
        except FileNotFoundError as err:
            full_reg_path = f"HKEY_LOCAL_MACHINE\\{key}"
            raise PhotoshopPythonAPIError(
                f"Failed to read the registration: <{full_reg_path}>, "
                "please check if you have Photoshop installed correctly."
            ) from err

    def get_application_path(self):
        """str: The absolute path of Photoshop installed location."""
        key = self.open_key(f"{self.REG_PATH}\\{self.get_program_id()}")
        return winreg.QueryValueEx(key, "ApplicationPath")[0]

    def get_plugin_path(self):
        """str: The absolute plugin path of Photoshop."""
        return os.path.join(self.get_application_path(), "Plug-ins")

    def get_presets_path(self):
        """str: The absolute presets path of Photoshop."""
        return os.path.join(self.get_application_path(), "Presets")

    def get_script_path(self):
        """str: The absolute scripts path of Photoshop."""
        return os.path.join(self.get_presets_path(), "Scripts")

    def instance_app(self, ps_id):
        naming_space = [self._root]
        if not self.object_name:
            naming_space.append(self._object_name)
        else:
            naming_space.append(self.object_name)
        naming_space.append(ps_id)
        self._program_name = self._assemble_program_name(naming_space)
        return CreateObject(self._program_name, dynamic=True)

    def get_program_id(self) -> str:
        key = self.open_key(self.REG_PATH)
        index = 0
        while True:
            value = winreg.EnumKey(key, index)
            index += 1
            if value:
                try:
                    self.instance_app(value.split(".")[0])
                except OSError:
                    continue
                return value.split(".")[0]

    def _get_program_id(self) -> str:
        self.app_id = self.get_program_id().split(".")[0]
        return self.app_id

    @staticmethod
    def _assemble_program_name(names: List[str]):
        """Assemble program name of Photoshop.

        Args:
            names (list of str): The name to be assembled.
                .e.g:
                    [
                        'Photoshop',
                        'ActionDescriptor',
                        '140'
                    ]

        Returns:
            str: Assembled name.

        Examples:
            Photoshop.ActionDescriptor
            Photoshop.ActionDescriptor.140
            Photoshop.ActionList
            Photoshop.ActionList.140
            Photoshop.ActionReference
            Photoshop.ActionReference.140
            Photoshop.Application
            Photoshop.Application.140
            Photoshop.BatchOptions
            Photoshop.BatchOptions.140
            Photoshop.BitmapConversionOptions
            Photoshop.BMPSaveOptions
            Photoshop.BMPSaveOptions.140
            Photoshop.CameraRAWOpenOptions
            Photoshop.CameraRAWOpenOptions.140

        """
        return ".".join(names)

    def eval_javascript(self, javascript, Arguments=None, ExecutionMode=None):
        executor = self.app
        if self._has_parent:
            executor = self.adobe
        return executor.doJavaScript(javascript, Arguments, ExecutionMode)



"""Defines a gray color, used in the `SolidColor` object."""
# Import local modules
from photoshop.api._core import Photoshop


class GrayColor(Photoshop):
    object_name = "GrayColor"

    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def gray(self) -> float:
        """The gray value."""
        return self.app.gray

    @gray.setter
    def gray(self, value: float):
        """The gray value."""
        self.app.gray = value



"""The Adobe Adobe Photoshop CC application object.

Which is the root of the object model and provides access to all other
objects. This object provides application-wide information,
such as application defaults and available fonts. It provides many important
methods, such as those for opening files and loading documents.

app = Application()

app.documents.add(800, 600, 72, "docRef")

"""

# Import built-in modules
import os
from pathlib import Path
import time
from typing import List

# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api._document import Document
from photoshop.api._documents import Documents
from photoshop.api._measurement_log import MeasurementLog
from photoshop.api._notifiers import Notifiers
from photoshop.api._preferences import Preferences
from photoshop.api._text_fonts import TextFonts
from photoshop.api.enumerations import DialogModes
from photoshop.api.solid_color import SolidColor


class Application(Photoshop):
    def __init__(self, version=None):
        super().__init__(ps_version=version)

    @property
    def activeLayer(self):
        return self.app.ArtLayer

    @property
    def layerSets(self):
        return self.app.LayerSets

    @property
    def activeDocument(self):
        """The frontmost documents.

        Setting this property is equivalent to clicking an
        open document in the Adobe Photoshop CC
        application to bring it to the front of the screen.

        """
        return Document(self.app.activeDocument)

    @activeDocument.setter
    def activeDocument(self, document):
        self.app.activeDocument = document

    @property
    def backgroundColor(self):
        """The default background color and color style for documents.

        Returns:
            .solid_color.SolidColor: The SolidColor instance.

        """
        return SolidColor(self.app.backgroundColor)

    @backgroundColor.setter
    def backgroundColor(self, color):
        """Sets the default background color and color style for documents.

        Args:
            color (.solid_color.SolidColor): The SolidColor instance.

        """
        self.app.backgroundColor = color

    @property
    def build(self):
        """str: The information about the application."""
        return self.app.build

    @property
    def colorSettings(self):
        """The name of the current color settings.

        as selected with Edit > Color Settings.

        """
        return self.app.colorSettings

    @colorSettings.setter
    def colorSettings(self, settings):
        """The name of the current color settings.

        Args:
            settings (str): The name of the current tool sel.

        """
        self.doJavaScript(f'app.colorSettings="{settings}"')

    @property
    def currentTool(self):
        """str: The name of the current tool sel."""
        return self.app.currentTool

    @currentTool.setter
    def currentTool(self, tool_name):
        """Sets the current tool for select.

        Args:
            tool_name (str): The name of the current tool sel.

        """
        self.app.currentTool = tool_name

    @property
    def displayDialogs(self) -> DialogModes:
        """The dialog mode for the document, which indicates whether or not
        Photoshop displays dialogs when the script runs."""
        return DialogModes(self.app.displayDialogs)

    @displayDialogs.setter
    def displayDialogs(self, dialog_mode: DialogModes):
        """The dialog mode for the document, which indicates whether or not
        Photoshop displays dialogs when the script runs.
        """
        self.app.displayDialogs = dialog_mode

    @property
    def documents(self) -> Documents:
        """._documents.Documents: The Documents instance."""
        return Documents(self.app.documents)

    @property
    def fonts(self) -> TextFonts:
        return TextFonts(self.app.fonts)

    @property
    def foregroundColor(self):
        """Get default foreground color.

        Used to paint, fill, and stroke selections.

        Returns:
            .solid_color.SolidColor: The SolidColor instance.

        """
        return SolidColor(parent=self.app.foregroundColor)

    @foregroundColor.setter
    def foregroundColor(self, color: SolidColor):
        """Set the `foregroundColor`.

        Args:
            color (.solid_color.SolidColor): The SolidColor instance.

        """
        self.app.foregroundColor = color

    @property
    def freeMemory(self) -> float:
        """The amount of unused memory available to ."""
        return self.app.freeMemory

    @property
    def locale(self) -> str:
        """The language locale of the application."""
        return self.app.locale

    @property
    def macintoshFileTypes(self) -> List[str]:
        """A list of the image file types Photoshop can open."""
        return self.app.macintoshFileTypes

    @property
    def measurementLog(self):
        """The log of measurements taken."""
        return MeasurementLog(self.app.measurementLog)

    @property
    def name(self) -> str:
        return self.app.name

    @property
    def notifiers(self):
        """The notifiers currently configured (in the Scripts Events Manager
        menu in the application)."""
        return Notifiers(self.app.notifiers)

    @property
    def notifiersEnabled(self):
        """If true, notifiers are enabled."""
        return self.app.notifiersEnabled

    @notifiersEnabled.setter
    def notifiersEnabled(self, value):
        self.app.notifiersEnabled = value

    @property
    def parent(self):
        """The object’s container."""
        return self.app.parent

    @property
    def path(self):
        """str: The full path to the location of the Photoshop application."""
        return Path(self.app.path)

    @property
    def playbackDisplayDialogs(self):
        return self.doJavaScript("app.playbackDisplayDialogs")

    @property
    def playbackParameters(self):
        """Stores and retrieves parameters used as part of a recorded action."""
        return self.app.playbackParameters

    @playbackParameters.setter
    def playbackParameters(self, value):
        self.app.playbackParameters = value

    @property
    def preferences(self):
        return Preferences(self.app.preferences)

    @property
    def preferencesFolder(self):
        return Path(self.app.preferencesFolder)

    @property
    def recentFiles(self):
        return self.app.recentFiles

    @property
    def scriptingBuildDate(self):
        return self.app.scriptingBuildDate

    @property
    def scriptingVersion(self):
        return self.app.scriptingVersion

    @property
    def systemInformation(self):
        return self.app.systemInformation

    @property
    def version(self):
        return self.app.version

    @property
    def windowsFileTypes(self):
        return self.app.windowsFileTypes

    # Methods.
    def batch(self, *args, **kwargs):
        """Runs the batch automation routine.

        Similar to the **File** > **Automate** > **Batch** command.

        """
        self.app.bath(*args, **kwargs)

    def beep(self):
        """Causes a "beep" sound."""
        return self.eval_javascript("app.beep()")

    def bringToFront(self):
        return self.eval_javascript("app.bringToFront()")

    def changeProgressText(self, text):
        """Changes the text that appears in the progress window."""
        self.eval_javascript(f"app.changeProgressText('{text}')")

    def charIDToTypeID(self, char_id):
        return self.app.charIDToTypeID(char_id)

    @staticmethod
    def compareWithNumbers(first, second):
        return first > second

    def doAction(self, action, action_from):
        """Plays the specified action from the Actions palette."""
        self.app.doAction(action, action_from)
        return True

    def doForcedProgress(self, title, javascript):
        script = "app.doForcedProgress('{}', '{}')".format(
            title,
            javascript,
        )
        self.eval_javascript(script)
        # Ensure the script execute success.
        time.sleep(1)

    def doProgress(self, title, javascript):
        """Performs a task with a progress bar. Other progress APIs must be
        called periodically to update the progress bar and allow cancelling.

        Args:
            title (str): String to show in the progress window.
            javascript (str): JavaScriptString to execute.

        """
        script = "app.doProgress('{}', '{}')".format(
            title,
            javascript,
        )
        self.eval_javascript(script)
        # Ensure the script execute success.
        time.sleep(1)

    def doProgressSegmentTask(self, segmentLength, done, total, javascript):
        script = "app.doProgressSegmentTask({}, {}, {}, '{}');".format(
            segmentLength,
            done,
            total,
            javascript,
        )
        self.eval_javascript(script)
        # Ensure the script execute success.
        time.sleep(1)

    def doProgressSubTask(self, index, limit, javascript):
        script = "app.doProgressSubTask({}, {}, '{}');".format(
            index,
            limit,
            javascript,
        )
        self.eval_javascript(script)
        # Ensure the script execute success.
        time.sleep(1)

    def doProgressTask(self, index, javascript):
        """Sections off a portion of the unused progress bar for execution of
        a subtask. Returns false on cancel.

        """
        script = f"app.doProgressTask({index}, '{javascript}');"
        self.eval_javascript(script)
        # Ensure the script execute success.
        time.sleep(1)

    def eraseCustomOptions(self, key):
        """Removes the specified user objects from the Photoshop registry."""
        self.app.eraseCustomOptions(key)

    def executeAction(self, event_id, descriptor, display_dialogs=2):
        return self.app.executeAction(event_id, descriptor, display_dialogs)

    def executeActionGet(self, reference):
        return self.app.executeActionGet(reference)

    def featureEnabled(self, name):
        """Determines whether the feature

        specified by name is enabled.
        The following features are supported

        as values for name:
        "photoshop/extended"
        "photoshop/standard"
        "photoshop/trial

        """
        return self.app.featureEnabled(name)

    def getCustomOptions(self, key):
        """Retrieves user objects in the Photoshop registry for the ID with
        value key."""
        return self.app.getCustomOptions(key)

    def open(
        self,
        document_file_path,
        document_type: str = None,
        as_smart_object: bool = False,
    ) -> Document:
        document = self.app.open(document_file_path, document_type, as_smart_object)
        if not as_smart_object:
            return Document(document)
        return document

    def load(self, document_file_path):
        """Loads a support document."""
        self.app.load(document_file_path)
        return self.activeDocument

    def doJavaScript(self, javascript, Arguments=None, ExecutionMode=None):
        return self.app.doJavaScript(javascript, Arguments, ExecutionMode)

    def isQuicktimeAvailable(self) -> bool:
        return self.app.isQuicktimeAvailable

    def openDialog(self):
        return self.app.openDialog()

    def purge(self, target):
        """Purges one or more caches.

        Args:
            target:
                .e.g:
                    0: Clears all caches.
                    1: Clears the clipboard.
                    2: Deletes all history states from the History palette.
                    3: Clears the undo cache.

        Returns:

        """
        self.app.purge(target)

    def putCustomOptions(self, key, custom_object, persistent):
        self.app.putCustomOptions(key, custom_object, persistent)

    def refresh(self):
        """Pauses the script while the application refreshes.

        Ues to slow down execution and show the results to the user as the
        script runs.
        Use carefully; your script runs much more slowly when using this
        method.

        """
        self.app.refresh()

    def refreshFonts(self):
        """Force the font list to get refreshed."""
        return self.eval_javascript("app.refreshFonts();")

    def runMenuItem(self, menu_id):
        """Run a menu item given the menu ID."""
        return self.eval_javascript(
            f"app.runMenuItem({menu_id})",
        )

    def showColorPicker(self):
        """Returns false if dialog is cancelled, true otherwise."""
        return self.eval_javascript("app.showColorPicker();")

    def stringIDToTypeID(self, string_id):
        return self.app.stringIDToTypeID(string_id)

    def togglePalettes(self):
        """Toggle palette visibility."""
        return self.doJavaScript("app.togglePalettes()")

    def toolSupportsBrushes(self, tool):
        return self.app.toolSupportsBrushes(tool)

    def toolSupportsBrushPresets(self, tool):
        return self.app.toolSupportsPresets(tool)

    @staticmethod
    def system(command):
        os.system(command)

    def typeIDToStringID(self, type_id):
        return self.app.typeIDToStringID(type_id)

    def typeIDToCharID(self, type_id):
        return self.app.typeIDToCharID(type_id)

    def updateProgress(self, done, total):
        self.eval_javascript(f"app.updateProgress({done}, {total})")



# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api.enumerations import MatteType


class JPEGSaveOptions(Photoshop):
    object_name = "JPEGSaveOptions"

    def __init__(self, quality=5, embedColorProfile=True, matte=MatteType.NoMatte):
        super().__init__()
        self.quality = quality
        self.embedColorProfile = embedColorProfile
        self.matte = matte

    @property
    def quality(self):
        return self.app.quality

    @quality.setter
    def quality(self, value):
        self.app.quality = value

    @property
    def formatOptions(self):
        """The download format to use."""
        return self.app.formatOptions

    @formatOptions.setter
    def formatOptions(self, value):
        self.app.formatOptions = value

    @property
    def embedColorProfile(self):
        return self.app.embedColorProfile

    @embedColorProfile.setter
    def embedColorProfile(self, value):
        self.app.embedColorProfile = value

    @property
    def matte(self):
        """The color to use to fill anti-aliased edges adjacent to
        transparent"""
        return self.app.matte

    @matte.setter
    def matte(self, value):
        self.app.matte = value

    @property
    def scans(self):
        return self.app.scans

    @scans.setter
    def scans(self, value):
        self.app.scans = value



# Import local modules
from photoshop.api._core import Photoshop


class MeasurementLog(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    def exportMeasurements(self, file_path: str, range_: int = None, data_point=None):
        if data_point is None:
            data_point = []
        self.app.exportMeasurements(file_path, range_, data_point)

    def deleteMeasurements(self, range_: int):
        self.app.deleteMeasurements(range_)



# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api.errors import PhotoshopPythonAPIError
from photoshop.api.text_font import TextFont


class TextFonts(Photoshop):
    def __init__(self, parent=None):
        super().__init__(parent=parent)

    def __iter__(self):
        for font in self.app:
            yield TextFont(font)

    @property
    def _fonts(self):
        return [a for a in self.app]

    def __len__(self):
        return self.length

    @property
    def length(self):
        """The number pf elements in the collection."""
        return len(self._fonts)

    def getByName(self, name):
        """Gets the font by the font name.

        Args:
            name (str): The name of the font.


        Returns:
            Font

        """
        for font in self.app:
            if font.name == name:
                return TextFont(font)
        raise PhotoshopPythonAPIError('Could not find a TextFont named "{name}"')



"""Provides a public session class for Photoshop api.

Usually we only need to manipulate the currently active document of photoshop.
So as follows:

.. code-block:: python

    from photoshop import Session

    with Session(action="new_document") as ps:
        doc = ps.active_document
        text_color = ps.SolidColor()
        text_color.rgb.green = 255
        new_text_layer = doc.artLayers.add()
        new_text_layer.kind = ps.LayerKind.TextLayer
        new_text_layer.textItem.contents = 'Hello, World!'
        new_text_layer.textItem.position = [160, 167]
        new_text_layer.textItem.size = 40
        new_text_layer.textItem.color = text_color
        options = ps.JPEGSaveOptions(quality=5)
        jpg = 'd:/hello_world.jpg'
        doc.saveAs(jpg, options, asCopy=True)
        ps.app.doJavaScript(f'alert("save to jpg: {jpg}")')

"""

# Import built-in modules
from typing import Any

# Import local modules
from photoshop.api import ActionDescriptor
from photoshop.api import ActionReference
from photoshop.api import Application
from photoshop.api import BMPSaveOptions
from photoshop.api import CMYKColor
from photoshop.api import EventID
from photoshop.api import ExportOptionsSaveForWeb
from photoshop.api import GIFSaveOptions
from photoshop.api import GrayColor
from photoshop.api import HSBColor
from photoshop.api import JPEGSaveOptions
from photoshop.api import LabColor
from photoshop.api import PDFSaveOptions
from photoshop.api import PNGSaveOptions
from photoshop.api import PhotoshopSaveOptions
from photoshop.api import RGBColor
from photoshop.api import SolidColor
from photoshop.api import TargaSaveOptions
from photoshop.api import TextItem
from photoshop.api import TiffSaveOptions
from photoshop.api import enumerations
from photoshop.api import errors


# pylint: disable=too-many-arguments
class Session:
    """Session of photoshop.

    We can control active documents in this Session.

    Attributes:
        app (photoshop.application.Application):

    """

    def __init__(
        self,
        file_path: str = None,
        action: str = None,
        callback: Any = None,
        auto_close: bool = False,
        ps_version: str = None,
    ):
        """Session of Photoshop.


        Examples:
            .. code-block:: python

                from photoshop import Session
                with Session("your/psd/or/psb/file_path.psd",
                            action="open") as ps:
                    ps.echo(ps.active_document.name)

        Args:
            file_path: The absolute path of the file. This path can be
                used together with action. If the path is an existing ``psd`
                or image path, use ``open`` action to open this file in the
                current session.
            action: Name of the action.
                .e.g:
                    - open
                        Open the file from the option `file_path`.
                    - new_document
                        Create a new document.
                    - document_duplicate
                        Duplicate current active document.
            callback: The callback function for this Photoshop
                session. The idea behind it is to allow us to pass some custom
                callback function every time we exit the current Photoshop
                session.
            auto_close: Is it necessary to close the current document
                when exiting the current context session. The default is
                ``False`` not to exit current session.
            ps_version: Specify the version number of photoshop.
                .e.g:
                    - 2022
                    - 2021
                    - cs6

        """
        super().__init__()

        self.path = file_path
        self._auto_close = auto_close
        self._callback = callback
        self._action = action
        self._active_document = None

        self.app = Application(version=ps_version)
        self.ActionReference = ActionReference()
        self.ActionDescriptor = ActionDescriptor()
        self.EventID = EventID
        self.SolidColor = SolidColor
        self.TextItem = TextItem

        # The save options.
        self.GIFSaveOptions = GIFSaveOptions
        self.JPEGSaveOptions = JPEGSaveOptions
        self.PDFSaveOptions = PDFSaveOptions
        self.PNGSaveOptions = PNGSaveOptions
        self.PhotoshopSaveOptions = PhotoshopSaveOptions
        self.ExportOptionsSaveForWeb = ExportOptionsSaveForWeb
        self.BMPSaveOptions = BMPSaveOptions
        self.TiffSaveOptions = TiffSaveOptions
        self.TargaSaveOptions = TargaSaveOptions

        # The colors.
        self.LabColor = LabColor
        self.HSBColor = HSBColor
        self.CMYKColor = CMYKColor
        self.RGBColor = RGBColor
        self.GrayColor = GrayColor

        # From enumerations
        self.LensType = enumerations.LensType
        self.AdjustmentReference = enumerations.AdjustmentReference
        self.AnchorPosition = enumerations.AnchorPosition
        self.AntiAlias = enumerations.AntiAlias
        self.AutoKernType = enumerations.AutoKernType
        self.BMPDepthType = enumerations.BMPDepthType
        self.BatchDestinationType = enumerations.BatchDestinationType
        self.BitmapConversionType = enumerations.BitmapConversionType
        self.BitmapHalfToneType = enumerations.BitmapHalfToneType
        self.BitsPerChannelType = enumerations.BitsPerChannelType
        self.BlendMode = enumerations.BlendMode
        self.ByteOrderType = enumerations.ByteOrderType
        self.CameraRAWSettingsType = enumerations.CameraRAWSettingsType
        self.CameraRAWSize = enumerations.CameraRAWSize
        self.Case = enumerations.Case
        self.ChangeMode = enumerations.ChangeMode
        self.ChannelType = enumerations.ChannelType
        self.ColorBlendMode = enumerations.ColorBlendMode
        self.ColorModel = enumerations.ColorModel
        self.ColorPicker = enumerations.ColorPicker
        self.ColorProfileType = enumerations.ColorProfileType
        self.ColorReductionType = enumerations.ColorReductionType
        self.ColorSpaceType = enumerations.ColorSpaceType
        self.CopyrightedType = enumerations.CopyrightedType
        self.CreateFields = enumerations.CreateFields
        self.CropToType = enumerations.CropToType
        self.DCSType = enumerations.DCSType
        self.DepthMaource = enumerations.DepthMaource
        self.DescValueType = enumerations.DescValueType
        self.DialogModes = enumerations.DialogModes
        self.Direction = enumerations.Direction
        self.DisplacementMapType = enumerations.DisplacementMapType
        self.DitherType = enumerations.DitherType
        self.DocumentFill = enumerations.DocumentFill
        self.DocumentMode = enumerations.DocumentMode
        self.EditLogItemsType = enumerations.EditLogItemsType
        self.ElementPlacement = enumerations.ElementPlacement
        self.EliminateFields = enumerations.EliminateFields
        self.ExportType = enumerations.ExportType
        self.ExtensionType = enumerations.ExtensionType
        self.FileNamingType = enumerations.FileNamingType
        self.FontPreviewType = enumerations.FontPreviewType
        self.ForcedColors = enumerations.ForcedColors
        self.FormatOptionsType = enumerations.FormatOptionsType
        self.GalleryConstrainType = enumerations.GalleryConstrainType
        self.GalleryFontType = enumerations.GalleryFontType
        self.GallerySecurityTextColorType = enumerations.GallerySecurityTextColorType
        self.GallerySecurityTextPositionType = (
            enumerations.GallerySecurityTextPositionType
        )
        self.GallerySecurityTextRotateType = enumerations.GallerySecurityTextRotateType
        self.GallerySecurityType = enumerations.GallerySecurityType
        self.GalleryThumbSizeType = enumerations.GalleryThumbSizeType
        self.Geometry = enumerations.Geometry
        self.GridLineStyle = enumerations.GridLineStyle
        self.GridSize = enumerations.GridSize
        self.GuideLineStyle = enumerations.GuideLineStyle
        self.IllustratorPathType = enumerations.IllustratorPathType
        self.Intent = enumerations.Intent
        self.JavaScriptExecutionMode = enumerations.JavaScriptExecutionMode
        self.Justification = enumerations.Justification
        self.Language = enumerations.Language
        self.LayerCompressionType = enumerations.LayerCompressionType
        self.LayerKind = enumerations.LayerKind
        self.LayerType = enumerations.LayerType
        self.MagnificationType = enumerations.MagnificationType
        self.MatteType = enumerations.MatteType
        self.MeasurementRange = enumerations.MeasurementRange
        self.MeasurementSource = enumerations.MeasurementSource
        self.NewDocumentMode = enumerations.NewDocumentMode
        self.NoiseDistribution = enumerations.NoiseDistribution
        self.OffsetUndefinedAreas = enumerations.OffsetUndefinedAreas
        self.OpenDocumentMode = enumerations.OpenDocumentMode
        self.OpenDocumentType = enumerations.OpenDocumentType
        self.OperatingSystem = enumerations.OperatingSystem
        self.Orientation = enumerations.Orientation
        self.OtherPaintingCursors = enumerations.OtherPaintingCursors
        self.PDFCompatibilityType = enumerations.PDFCompatibilityType
        self.PDFEncodingType = enumerations.PDFEncodingType
        self.PDFResampleType = enumerations.PDFResampleType
        self.PDFStandardType = enumerations.PDFStandardType
        self.PICTBitsPerPixel = enumerations.PICTBitsPerPixel
        self.PICTCompression = enumerations.PICTCompression
        self.PaintingCursors = enumerations.PaintingCursors
        self.PaletteType = enumerations.PaletteType
        self.PathKind = enumerations.PathKind
        self.PhotoCDColorSpace = enumerations.PhotoCDColorSpace
        self.PhotoCDSize = enumerations.PhotoCDSize
        self.PicturePackageTextType = enumerations.PicturePackageTextType
        self.PointKind = enumerations.PointKind
        self.PointType = enumerations.PointType
        self.PolarConversionType = enumerations.PolarConversionType
        self.PreviewType = enumerations.PreviewType
        self.PurgeTarget = enumerations.PurgeTarget
        self.QueryStateType = enumerations.QueryStateType
        self.RadialBlurMethod = enumerations.RadialBlurMethod
        self.RadialBlurBest = enumerations.RadialBlurBest
        self.RasterizeType = enumerations.RasterizeType
        self.ReferenceFormType = enumerations.ReferenceFormType
        self.ResampleMethod = enumerations.ResampleMethod
        self.ResetTarget = enumerations.ResetTarget
        self.RippleSize = enumerations.RippleSize
        self.SaveBehavior = enumerations.SaveBehavior
        self.SaveDocumentType = enumerations.SaveDocumentType
        self.SaveEncoding = enumerations.SaveEncoding
        self.SaveLogItemsType = enumerations.SaveLogItemsType
        self.SaveOptions = enumerations.SaveOptions
        self.SelectionType = enumerations.SelectionType
        self.ShapeOperation = enumerations.ShapeOperation
        self.SmartBlurMode = enumerations.SmartBlurMode
        self.SmartBlurQuality = enumerations.SmartBlurQuality
        self.SourceSpaceType = enumerations.SourceSpaceType
        self.SpherizeMode = enumerations.SpherizeMode
        self.StrikeThruType = enumerations.StrikeThruType
        self.StrokeLocation = enumerations.StrokeLocation
        self.TargaBitsPerPixels = enumerations.TargaBitsPerPixels
        self.TextComposer = enumerations.TextComposer
        self.TextType = enumerations.TextType
        self.TextureType = enumerations.TextureType
        self.TiffEncodingType = enumerations.TiffEncodingType
        self.ToolType = enumerations.ToolType
        self.TransitionType = enumerations.TransitionType
        self.TrimType = enumerations.TrimType
        self.TypeUnits = enumerations.TypeUnits
        self.UndefinedAreas = enumerations.UndefinedAreas
        self.UnderlineType = enumerations.UnderlineType
        self.Units = enumerations.Units
        self.Urgency = enumerations.Urgency
        self.Wartyle = enumerations.Wartyle
        self.WaveType = enumerations.WaveType
        self.WhiteBalanceType = enumerations.WhiteBalanceType
        self.ZigZagType = enumerations.ZigZagType

    @property
    def active_document(self):
        try:
            if not self._active_document:
                return self.app.activeDocument
            return self._active_document
        except errors.PhotoshopPythonAPICOMError:
            raise errors.PhotoshopPythonAPIError("No active document " "available.")

    @staticmethod
    def echo(*args, **kwargs):
        """Print message."""
        print(*args, **kwargs)

    def alert(self, text: str):
        """Alert message box in photoshop.

        Args:
            text (str): The text will pop up in photoshop.

        """
        self.app.doJavaScript(f"alert('{text}')")

    @active_document.setter
    def active_document(self, active_document):
        """Set active document."""
        self._active_document = active_document

    def _action_open(self):
        self.active_document = self.app.open(self.path)

    def _action_new_document(self):
        self.active_document = self.app.documents.add()

    def _action_document_duplicate(self):
        self.active_document = self.active_document.duplicate()

    def run_action(self):
        try:
            _action = getattr(self, f"_action_{self._action}")
            _action()
        except AttributeError:
            pass

    def close(self):
        """closing current session."""
        if self._auto_close:
            self.active_document.close()

    def __enter__(self):
        self.run_action()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        try:
            if self._callback:
                self._callback(self)
        except Exception as err:
            raise errors.PhotoshopPythonAPIError(err)
        finally:
            self.close()



# Import local modules
from photoshop.api._core import Photoshop


class LabColor(Photoshop):
    object_name = "LabColor"

    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def A(self):
        return round(self.app.A)

    @A.setter
    def A(self, value):
        self.app.A = value

    @property
    def B(self):
        return round(self.app.B)

    @B.setter
    def B(self, value):
        self.app.B = value

    @property
    def L(self):
        return round(self.app.L)

    @L.setter
    def L(self, value):
        self.app.L = value



"""
The collection of Notifier objects in the document. Access through the
Application.notifiers collection property. For example:
var notRef = app.notifiers.add("OnClickGoButton", eventFile)
Notifiers must be enabled using the Application.notifiersEnabled property

"""
# Import built-in modules
from pathlib import Path

# Import local modules
from photoshop.api._core import Photoshop


class Notifier(Photoshop):
    def __init__(self, parent=None):
        print(parent)
        super().__init__()

    @property
    def event(self):
        """The event identifier, a four-character code or a unique string."""
        return self.app.event

    @property
    def eventClass(self):
        """The class identifier, a four-character code or a unique string.

        When an event applies to multiple types of objects, use this
        propery to distinguish which object this notifier applies to.
        For example, the Make event ("Mk ") can apply to
        documents ("Dcmn"), channels ("Chnl") and other objects.

        """
        return self.app.eventClass

    @property
    def eventFile(self) -> Path:
        """The path to the file to execute when the event occurs and
        activates the notifier."""
        return Path(self.app.eventFile)

    def remove(self):
        """Deletes this object.

        You can also remove a Notifier object
        from the Script Events Manager
        drop-down list by deleting the file named
        Script Events Manager.xml from the
        Photoshop preferences folder. See Adobe
        Photoshop CC help for more information.

        """
        return self.app.remove()



"""Options for saving a document in Adobe PDF format.

using the Document.saveAs() method.

"""

# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api.enumerations import PDFEncodingType
from photoshop.api.enumerations import PDFResampleType
from photoshop.api.errors import COMError


# pylint: disable=too-many-instance-attributes,too-many-public-methods
class PDFSaveOptions(Photoshop):
    object_name = "PDFSaveOptions"

    def __init__(self, **kwargs):
        super().__init__()
        self.layers = False
        self.jpegQuality = 12
        self.alphaChannels = False
        self.embedThumbnail = True
        self.view = False
        self.annotations = True
        self.colorConversion = False
        self.convertToEightBit = True
        self.description = "No description."
        self.encoding_types = PDFEncodingType
        self.downSample = PDFResampleType.NoResample
        self.embedColorProfile = True
        if kwargs:
            if "encoding" in kwargs:
                self.encoding = kwargs.get("encoding", self.encoding_types.PDFJPEG)
            for key, value in kwargs.items():
                setattr(self, key, value)

    @property
    def alphaChannels(self):
        """True to save the alpha channels with the file."""
        return self.app.alphaChannels

    @alphaChannels.setter
    def alphaChannels(self, value):
        """True to save the alpha channels with the file."""
        self.app.alphaChannels = value

    @property
    def annotations(self):
        """If true, the annotations are saved."""
        return self.app.anotations

    @annotations.setter
    def annotations(self, value):
        """If true, the annotations are saved."""
        self.app.annotations = value

    @property
    def colorConversion(self):
        """If true, converts the color profile to the destination profile."""
        return self.app.colorConversion

    @colorConversion.setter
    def colorConversion(self, value):
        """If true, converts the color profile to the destination profile."""
        self.app.colorConversion = value

    @property
    def convertToEightBit(self):
        """If true, converts a 16-bit image to 8-bit for better
        compatibility with other applications."""
        return self.app.convertToEightBit

    @convertToEightBit.setter
    def convertToEightBit(self, value):
        """If true, converts a 16-bit image to 8-bit for better
        compatibility with other applications."""
        self.app.convertToEightBit = value

    @property
    def description(self):
        """Description of the save options in use."""
        return self.app.description

    @description.setter
    def description(self, text):
        """Description of the save options in use."""
        self.app.description = text

    @property
    def destinationProfile(self):
        """Describes the final RGB or CMYK output device,
        such as a monitor or press standard."""
        try:
            return self.app.destinationProfile
        except COMError:
            raise ValueError(
                "Should set value first. "
                "This parameter can only be read after the "
                "value has been set."
            )

    @destinationProfile.setter
    def destinationProfile(self, value):
        """Describes the final RGB or CMYK output device,
        such as a monitor or press standard."""
        self.app.destinationProfile = value

    @property
    def downSample(self):
        """The downsample method to use."""
        return self.app.downSample

    @downSample.setter
    def downSample(self, value):
        """The downsample method to use."""
        self.app.downSample = value

    @property
    def downSampleSize(self):
        """The size (in pixels per inch) to downsample images to if they
        exceed the value specified for down sample size limit."""
        try:
            return self.app.downSampleSize
        except COMError:
            raise ValueError(
                "Should set value first. "
                "This parameter can only be read after the "
                "value has been set."
            )

    @downSampleSize.setter
    def downSampleSize(self, value):
        """The size (in pixels per inch) to downsample images to if they
        exceed the value specified for ‘down sample size limit’."""
        self.app.downSampleSize = value

    @property
    def downSampleSizeLimit(self):
        """Limits downsampling or subsampling to images that
        exceed this value (in pixels per inch)."""
        try:
            return self.app.downSampleSizeLimit
        except COMError:
            raise ValueError(
                "Should set value first. "
                "This parameter can only be read after the "
                "value has been set."
            )

    @downSampleSizeLimit.setter
    def downSampleSizeLimit(self, value: float):
        """Limits downsampling or subsampling to images that exceed this
        value (in pixels per inch)."""
        self.app.downSampleSizeLimit = value

    @property
    def embedColorProfile(self):
        """If true, the color profile is embedded in the document."""
        return self.app.embedColorProfile

    @embedColorProfile.setter
    def embedColorProfile(self, value: bool):
        """If true, the color profile is embedded in the document."""
        self.app.embedColorProfile = value

    @property
    def embedThumbnail(self):
        """If true, includes a small preview image in Acrobat."""
        return self.app.embedThumbnail

    @embedThumbnail.setter
    def embedThumbnail(self, value: bool):
        """If true, includes a small preview image in Acrobat."""
        self.app.embedThumbnail = value

    @property
    def encoding(self):
        """The encoding method to use."""
        try:
            return self.app.encoding
        except COMError:
            return self.encoding_types.PDFJPEG

    @encoding.setter
    def encoding(self, value: str):
        """The encoding method to use."""
        self.app.encoding = value

    @property
    def jpegQuality(self):
        """Get the quality of the produced image."""
        return self.app.jpegQuality

    @jpegQuality.setter
    def jpegQuality(self, quality: int):
        """Set the quality of the produced image.

        Valid only for JPEG-encoded PDF documents. Range: 0 to 12.

        """
        self.app.jpegQuality = quality

    @property
    def layers(self):
        """If true, the layers are saved."""
        return self.app.layers

    @layers.setter
    def layers(self, value: bool):
        """If true, the layers are saved."""
        self.app.layers = value

    @property
    def optimizeForWeb(self):
        """If true, improves performance of PDFs on Web servers."""
        return self.app.optimizeForWeb

    @optimizeForWeb.setter
    def optimizeForWeb(self, value: bool):
        """If true, improves performance of PDFs on Web servers."""
        self.app.optimizeForWeb = value

    @property
    def outputCondition(self):
        """An optional comment field for inserting descriptions of the
        output condition. The text is stored in the PDF/X file."""
        return self.app.outputCondition

    @outputCondition.setter
    def outputCondition(self, value):
        """An optional comment field for inserting descriptions of
        the output condition. The text is stored in the PDF/X file."""
        self.app.outputCondition = value

    @property
    def outputConditionID(self):
        """The identifier for the output condition."""
        try:
            return self.app.outputConditionID
        except COMError:
            raise ValueError(
                "Should set value first. "
                "This parameter can only be read after the "
                "value has been set."
            )

    @outputConditionID.setter
    def outputConditionID(self, value):
        """The identifier for the output condition."""
        self.app.outputConditionID = value

    @property
    def preserveEditing(self):
        """If true, allows users to reopen the PDF in Photoshop with
        native Photoshop data intact."""
        try:
            return self.app.preserveEditing
        except COMError:
            raise ValueError(
                "Should set value first. "
                "This parameter can only be read after the "
                "value has been set."
            )

    @preserveEditing.setter
    def preserveEditing(self, value):
        """If true, allows users to reopen the PDF in Photoshop with
        native Photoshop data intact."""
        self.app.preserveEditing = value

    @property
    def presetFile(self):
        """The preset file to use for settings; overrides other settings."""
        try:
            return self.app.presetFile
        except COMError:
            raise ValueError(
                "Should set value first. "
                "This parameter can only be read after the "
                "value has been set."
            )

    @presetFile.setter
    def presetFile(self, file_name):
        """The preset file to use for settings; overrides other settings."""
        self.app.presetFile = file_name

    @property
    def profileInclusionPolicy(self):
        """If true, shows which profiles to include."""
        try:
            return self.app.profileInclusionPolicy
        except COMError:
            raise ValueError(
                "Should set value first. "
                "This parameter can only be read after the "
                "value has been set."
            )

    @profileInclusionPolicy.setter
    def profileInclusionPolicy(self, value):
        """If true, shows which profiles to include."""
        self.app.profileInclusionPolicy = value

    @property
    def registryName(self):
        """The URL where the output condition is registered."""
        try:
            return self.app.registryName
        except COMError:
            raise ValueError(
                "Should set value first. "
                "This parameter can only be read after the "
                "value has been set."
            )

    @registryName.setter
    def registryName(self, value):
        """The URL where the output condition is registered."""
        self.app.registryName = value

    @property
    def spotColors(self):
        """If true, the spot colors are saved."""
        try:
            return self.app.spotColors
        except COMError:
            raise ValueError(
                "Should set value first. "
                "This parameter can only be read after the "
                "value has been set."
            )

    @spotColors.setter
    def spotColors(self, value):
        """If true, the spot colors are saved."""
        self.app.spotColors = value

    @property
    def tileSize(self):
        """The compression option. Valid only when encoding is JPEG2000."""
        try:
            return self.app.tileSize
        except COMError:
            raise ValueError(
                "Should set value first. "
                "This parameter can only be read after the "
                "value has been set."
            )

    @tileSize.setter
    def tileSize(self, value):
        """The compression option. Valid only when encoding is JPEG2000."""
        if self.encoding not in (
            self.encoding_types.PDFJPEG2000HIGH,
            self.encoding_types.PDFJPEG2000LOSSLESS,
            self.encoding_types.PDFJPEG2000MED,
            self.encoding_types.PDFJPEG2000MEDLOW,
            self.encoding_types.PDFJPEG2000LOW,
            self.encoding_types.PDFJPEG2000MEDHIGH,
        ):
            raise ValueError(
                "tileSize only work in JPEG2000. Please "
                "change PDFSaveOptions.encoding to JPEG2000."
            )
        self.app.tileSize = value

    @property
    def view(self):
        """If true, opens the saved PDF in Acrobat."""
        return self.app.view

    @view.setter
    def view(self, value):
        """If true, opens the saved PDF in Acrobat."""
        self.app.view = value



"""Options for saving a document in BMO format."""
# Import local modules
from photoshop.api._core import Photoshop


class BMPSaveOptions(Photoshop):
    object_name = "BMPSaveOptions"

    def __init__(self):
        super().__init__()

    @property
    def alphaChannels(self):
        """State to save the alpha channels."""
        return self.app.alphaChannels

    @alphaChannels.setter
    def alphaChannels(self, value):
        """Sets whether to save the alpha channels or not.

        Args:

        """
        self.app.alphaChannels = value



# Import built-in modules
from enum import Enum


class EventID(str, Enum):
    # Here is a list of JSON CallBack events in Photoshop.
    # https://community.adobe.com/t5/get-started/photoshop-json-callback-events-list-up-to-cc2015-ver-16/td-p/4792115?page=1
    TDTransform = "TdT "
    Average = "Avrg"
    ApplyStyle = "ASty"
    Assert = "Asrt"
    AccentedEdges = "AccE"
    Add = "Add"
    AddNoise = "AdNs"
    AddTo = "AddT"
    Align = "Algn"
    All = "All "
    AngledStrokes = "AngS"
    ApplyImage = "AppI"
    BasRelief = "BsRl"
    Batch = "Btch"
    BatchFromDroplet = "BtcF"
    Blur = "Blr "
    BlurMore = "BlrM"
    Border = "Brdr"
    Brightness = "BrgC"
    CanvasSize = "CnvS"
    ChalkCharcoal = "ChlC"
    ChannelMixer = "ChnM"
    Charcoal = "Chrc"
    Chrome = "Chrm"
    Clear = "Cler"
    Close = "Cls "
    Clouds = "Clds"
    ColorBalance = "ClrB"
    ColorHalftone = "ClrH"
    ColorRange = "ClrR"
    ColoredPencil = "ClrP"
    ContactSheet = "0B71D221-F8CE-11d2-B21B-0008C75B322C"
    ConteCrayon = "CntC"
    Contract = "Cntc"
    ConvertMode = "CnvM"
    Copy = "copy"
    CopyEffects = "CpFX"
    CopyMerged = "CpyM"
    CopyToLayer = "CpTL"
    Craquelure = "Crql"
    CreateDroplet = "CrtD"
    Crop = "Crop"
    Crosshatch = "Crsh"
    Crystallize = "Crst"
    Curves = "Crvs"
    Custom = "Cstm"
    Cut = "cut "
    CutToLayer = "CtTL"
    Cutout = "Ct  "
    DarkStrokes = "DrkS"
    DeInterlace = "Dntr"
    DefinePattern = "DfnP"
    Defringe = "Dfrg"
    Delete = "Dlt "
    Desaturate = "Dstt"
    Deselect = "Dslc"
    Despeckle = "Dspc"
    DifferenceClouds = "DrfC"
    Diffuse = "Dfs "
    DiffuseGlow = "DfsG"
    DisableLayerFX = "dlfx"
    Displace = "Dspl"
    Distribute = "Dstr"
    Draw = "Draw"
    DryBrush = "DryB"
    Duplicate = "Dplc"
    DustAndScratches = "DstS"
    Emboss = "Embs"
    Equalize = "Eqlz"
    Exchange = "Exch"
    Expand = "Expn"
    Export = "Expr"
    Jumpto = "Jpto"
    ExportTransparentImage = "02879e00-cb66-11d1-bc43-0060b0a13dc4"
    Extrude = "Extr"
    Facet = "Fct "
    Fade = "Fade"
    Feather = "Fthr"
    Fibers = "Fbrs"
    Fill = "Fl  "
    FilmGrain = "FlmG"
    Filter = "Fltr"
    FindEdges = "FndE"
    FitImage = "3caa3434-cb67-11d1-bc43-0060b0a13dc4"
    FlattenImage = "FltI"
    Flip = "Flip"
    Fragment = "Frgm"
    Fresco = "Frsc"
    GaussianBlur = "GsnB"
    Get = "getd"
    Glass = "Gls "
    GlowingEdges = "GlwE"
    Gradient = "Grdn"
    GradientMap = "GrMp"
    Grain = "Grn "
    GraphicPen = "GraP"
    Group = "GrpL"
    Grow = "Grow"
    HalftoneScreen = "HlfS"
    Hide = "Hd  "
    HighPass = "HghP"
    HSBHSL = "HsbP"
    HueSaturation = "HStr"
    ImageSize = "ImgS"
    Import = "Impr"
    InkOutlines = "InkO"
    Intersect = "Intr"
    IntersectWith = "IntW"
    Inverse = "Invs"
    Invert = "Invr"
    LensFlare = "LnsF"
    Levels = "Lvls"
    LightingEffects = "LghE"
    Link = "Lnk "
    Make = "Mk  "
    Maximum = "Mxm "
    Median = "Mdn "
    MergeLayers = "Mrg2"
    MergeLayersOld = "MrgL"
    MergeSpotChannel = "MSpt"
    MergeVisible = "MrgV"
    Mezzotint = "Mztn"
    Minimum = "Mnm "
    ModeChange = "8cba8cd6-cb66-11d1-bc43-0060b0a13dc4"
    Mosaic = "Msc "
    Mosaic_PLUGIN = "MscT"
    MotionBlur = "MtnB"
    Move = "move"
    NTSCColors = "NTSC"
    NeonGlow = "NGlw"
    Next = "Nxt "
    NotePaper = "NtPr"
    Notify = "Ntfy"
    Null = "typeNull"
    OceanRipple = "OcnR"
    Offset = "Ofst"
    Open = "Opn "
    Paint = "Pnt "
    PaintDaubs = "PntD"
    PaletteKnife = "PltK"
    Paste = "past"
    PasteEffects = "PaFX"
    PasteInto = "PstI"
    PasteOutside = "PstO"
    Patchwork = "Ptch"
    Photocopy = "Phtc"
    PicturePackage = "4C1ABF40-DD82-11d2-B20F-0008C75B322C"
    Pinch = "Pnch"
    Place = "Plc "
    Plaster = "Plst"
    PlasticWrap = "PlsW"
    Play = "Ply "
    Pointillize = "Pntl"
    Polar = "Plr "
    PosterEdges = "PstE"
    Posterize = "Pstr"
    Previous = "Prvs"
    Print = "Prnt"
    ProfileToProfile = "PrfT"
    Purge = "Prge"
    Quit = "quit"
    RadialBlur = "RdlB"
    Rasterize = "Rstr"
    RasterizeTypeSheet = "RstT"
    RemoveBlackMatte = "RmvB"
    RemoveLayerMask = "RmvL"
    RemoveWhiteMatte = "RmvW"
    Rename = "Rnm "
    ReplaceColor = "RplC"
    Reset = "Rset"
    ResizeImage = "1333cf0c-cb67-11d1-bc43-0060b0a13dc4"
    Reticulation = "Rtcl"
    Revert = "Rvrt"
    Ripple = "Rple"
    Rotate = "Rtte"
    RoughPastels = "RghP"
    Save = "save"
    Select = "slct"
    SelectiveColor = "SlcC"
    Set = "setd"
    SharpenEdges = "ShrE"
    Sharpen = "Shrp"
    SharpenMore = "ShrM"
    Shear = "Shr "
    Show = "Shw "
    Similar = "Smlr"
    SmartBlur = "SmrB"
    Smooth = "Smth"
    SmudgeStick = "SmdS"
    Solarize = "Slrz"
    Spatter = "Spt "
    Spherize = "Sphr"
    SplitChannels = "SplC"
    Sponge = "Spng"
    SprayedStrokes = "SprS"
    StainedGlass = "StnG"
    Stamp = "Stmp"
    Stop = "Stop"
    Stroke = "Strk"
    Subtract = "Sbtr"
    SubtractFrom = "SbtF"
    Sumie = "Smie"
    TakeMergedSnapshot = "TkMr"
    TakeSnapshot = "TkSn"
    TextureFill = "TxtF"
    Texturizer = "Txtz"
    Threshold = "Thrs"
    Tiles = "Tls "
    TornEdges = "TrnE"
    TraceContour = "TrcC"
    Transform = "Trnf"
    Trap = "Trap"
    Twirl = "Twrl"
    Underpainting = "Undr"
    Undo = "undo"
    Ungroup = "Ungr"
    Unlink = "Unlk"
    UnsharpMask = "UnsM"
    Variations = "Vrtn"
    Wait = "Wait"
    WaterPaper = "WtrP"
    Watercolor = "Wtrc"
    Wave = "Wave"
    Wind = "Wnd "
    ZigZag = "ZgZg"
    BackLight = "BacL"
    FillFlash = "FilE"
    ColorCast = "ColE"
    OpenUntitled = "OpnU"
    PresetKind = "presetKindType"
    SmartSharpen = "smartSharpen"
    PresetKindType = "presetKindType"
    PresetKindCustom = "presetKindCustom"
    NoiseReduction = "noiseReduction"
    BlurType = "blurType"
    ContentLayer = "contentLayer"
    SaveStage = "saveStage"
    SaveStageType = "saveStageType"
    SaveSucceeded = "saveSucceeded"
    RasterizeLayer = "rasterizeLayer"
    ForceNotify = "forceNotify"
    PlacedLayerEditContents = "placedLayerEditContents"



# Import local modules
from photoshop.api._core import Photoshop


class ActiveLayer(Photoshop):
    def __int__(self):
        super().__init__()

    @property
    def name(self) -> str:
        return self.active_layer.Typename

    def add(self):
        self.app.ActiveDocument.ArtLayers.Add()



# Import local modules
from photoshop.api.save_options.bmp import BMPSaveOptions
from photoshop.api.save_options.gif import GIFSaveOptions
from photoshop.api.save_options.jpg import JPEGSaveOptions
from photoshop.api.save_options.pdf import PDFSaveOptions
from photoshop.api.save_options.png import ExportOptionsSaveForWeb
from photoshop.api.save_options.png import PNGSaveOptions
from photoshop.api.save_options.psd import PhotoshopSaveOptions
from photoshop.api.save_options.tag import TargaSaveOptions
from photoshop.api.save_options.tif import TiffSaveOptions


__all__ = [
    "BMPSaveOptions",
    "GIFSaveOptions",
    "JPEGSaveOptions",
    "PDFSaveOptions",
    "ExportOptionsSaveForWeb",
    "PNGSaveOptions",
    "PhotoshopSaveOptions",
    "TiffSaveOptions",
    "TargaSaveOptions",
]



"""Metadata about a document object.

These values can be set by selecting File > File Info in the Adobe Photoshop
application.

"""

# Import built-in modules
from pprint import pformat

# Import local modules
from photoshop.api._core import Photoshop


# pylint: disable=too-many-public-methods
class DocumentInfo(Photoshop):
    """Metadata about a document object."""

    def __init__(self, parent):
        super().__init__(parent=parent)

    def __str__(self):
        return pformat(
            {
                "author": self.author,
                "authorPosition": self.authorPosition,
                "caption": self.caption,
                "captionWriter": self.captionWriter,
                "category": self.category,
                "city": self.city,
                "country": self.country,
                "copyrightNotice": self.copyrightNotice,
                "copyrighted": self.copyrighted,
                "creationDate": self.creationDate,
                "credit": self.credit,
                "exif": self.exif,
                "headline": self.headline,
                "instructions": self.instructions,
                "jobName": self.jobName,
                "keywords": self.keywords,
                "provinceState": self.provinceState,
                "source": self.source,
                "ownerUrl": self.ownerUrl,
                "supplementalCategories": self.supplementalCategories,
                "title": self.title,
                "transmissionReference": self.transmissionReference,
                "urgency": self.urgency,
            }
        )

    @property
    def author(self):
        """str: The author."""
        return self.app.author

    @author.setter
    def author(self, name):
        self.app.author = name

    @property
    def authorPosition(self):
        """str:The author’s position."""
        return self.app.authorPosition

    @authorPosition.setter
    def authorPosition(self, name):
        self.app.authorPosition = name

    @property
    def caption(self):
        return self.app.caption

    @caption.setter
    def caption(self, name):
        self.app.caption = name

    @property
    def captionWriter(self):
        return self.app.captionWriter

    @captionWriter.setter
    def captionWriter(self, name):
        self.app.captionWriter = name

    @property
    def category(self):
        """str: The document category."""
        return self.app.category

    @category.setter
    def category(self, name):
        self.app.category = name

    @property
    def city(self):
        return self.app.city

    @city.setter
    def city(self, city_name):
        self.app.city = city_name

    @property
    def copyrightNotice(self):
        """str: The copyright notice."""
        return self.app.copyrightNotice

    @copyrightNotice.setter
    def copyrightNotice(self, name):
        self.app.copyrightNotice = name

    @property
    def copyrighted(self):
        """str: The copyright status."""
        return self.app.copyrighted

    @copyrighted.setter
    def copyrighted(self, info):
        self.app.copyrighted = info

    @property
    def country(self):
        return self.app.country

    @country.setter
    def country(self, name):
        self.app.country = name

    @property
    def creationDate(self):
        return self.app.creationDate

    @creationDate.setter
    def creationDate(self, name):
        self.app.creationDate = name

    @property
    def credit(self):
        """str: The author credit."""
        return self.app.credit

    @credit.setter
    def credit(self, value):
        self.app.credit = value

    @property
    def exif(self):
        return self.app.exif

    @exif.setter
    def exif(self, info):
        self.app.exif = info

    @property
    def headline(self):
        return self.app.headline

    @headline.setter
    def headline(self, value):
        self.app.headline = value

    @property
    def instructions(self):
        return self.app.instructions

    @instructions.setter
    def instructions(self, value):
        self.app.instructions = value

    @property
    def jobName(self):
        return self.app.jobName

    @jobName.setter
    def jobName(self, job):
        self.app.jobName = job

    @property
    def keywords(self):
        return self.app.keywords

    @keywords.setter
    def keywords(self, words):
        self.app.keywords = words

    @property
    def ownerUrl(self):
        return self.app.ownerUrl

    @ownerUrl.setter
    def ownerUrl(self, url):
        self.app.ownerUrl = url

    @property
    def provinceState(self):
        """str: The state or province."""
        return self.app.provinceState

    @provinceState.setter
    def provinceState(self, state_name):
        self.app.provinceState = state_name

    @property
    def source(self):
        return self.app.source

    @source.setter
    def source(self, source_name):
        self.app.source = source_name

    @property
    def supplementalCategories(self):
        """str: Other categories."""
        return self.app.supplementalCategories

    @supplementalCategories.setter
    def supplementalCategories(self, info):
        self.app.supplementalCategories = info

    @property
    def title(self):
        return self.app.title

    @title.setter
    def title(self, name):
        self.app.title = name

    @property
    def transmissionReference(self):
        """str: The transmission reference."""
        return self.app.transmissionReference

    @transmissionReference.setter
    def transmissionReference(self, reference):
        self.app.transmissionReference = reference

    @property
    def urgency(self):
        """The document urgency."""
        return self.app.urgency

    @urgency.setter
    def urgency(self, status):
        self.app.urgency = status



# Import local modules
from photoshop.api._core import Photoshop


class PhotoshopSaveOptions(Photoshop):
    object_name = "PhotoshopSaveOptions"

    def __int__(self):
        super().__init__()

    @property
    def alphaChannels(self):
        """If true, the alpha channels are saved."""
        return self.app.alphaChannels()

    @alphaChannels.setter
    def alphaChannels(self, value):
        self.app.alphaChannels = value

    @property
    def annotations(self):
        """If true, the annotations are saved."""
        return self.app.annotations()

    @annotations.setter
    def annotations(self, value):
        self.app.annotations = value

    @property
    def embedColorProfile(self):
        """If true, the color profile is embedded in the document."""
        return self.app.embedColorProfile()

    @embedColorProfile.setter
    def embedColorProfile(self, value):
        self.app.embedColorProfile = value

    @property
    def layers(self):
        """If true, the layers are saved."""
        return self.app.layers()

    @layers.setter
    def layers(self, value):
        self.app.layers = value

    @property
    def spotColors(self):
        """If true, spot colors are saved."""
        return self.app.spotColors()

    @spotColors.setter
    def spotColors(self, value):
        self.app.spotColors = value



# Import local modules
from photoshop.api._core import Photoshop


class EPSSaveOptions(Photoshop):
    """Options for saving a document in EPS format.

    using the `Document.saveAs()`

    """

    object_name = "EPSSaveOptions"

    def __init__(self):
        super().__init__()

    @property
    def embedColorProfile(self) -> bool:
        """True to embed the color profile in this document."""
        return self.app.embedColorProfile

    @embedColorProfile.setter
    def embedColorProfile(self, boolean: bool):
        """True to embed the color profile in this document."""
        self.app.embedColorProfile = boolean

    @property
    def encoding(self):
        return self.app.encoding

    @encoding.setter
    def encoding(self, value: bool):
        self.app.encoding = value

    @property
    def halftoneScreen(self):
        return self.app.halftoneScreen

    @halftoneScreen.setter
    def halftoneScreen(self, value: bool):
        self.app.halftoneScreen = value

    @property
    def interpolation(self):
        return self.app.interpolation

    @interpolation.setter
    def interpolation(self, value: bool):
        self.app.interpolation = value

    @property
    def preview(self):
        return self.app.preview

    @preview.setter
    def preview(self, value: bool):
        self.app.preview = value

    @property
    def psColorManagement(self):
        return self.app.psColorManagement

    @psColorManagement.setter
    def psColorManagement(self, value: bool):
        self.app.psColorManagement = value

    @property
    def transferFunction(self):
        return self.app.transferFunction

    @transferFunction.setter
    def transferFunction(self, value: bool):
        self.app.transferFunction = value

    @property
    def transparentWhites(self) -> bool:
        """True to display white areas as transparent"""
        return self.app.transparentWhites

    @transparentWhites.setter
    def transparentWhites(self, value: bool):
        """True to display white areas as transparent"""
        self.app.transparentWhites = value

    @property
    def vectorData(self):
        """True to include vector data."""
        return self.app.vectorData

    @vectorData.setter
    def vectorData(self, value: bool):
        """True to include vector data.

        Valid only if the document includes vector data (text).

        """
        self.app.vectorData = value



# Import local modules
from photoshop.api._core import Photoshop


class TextFont(Photoshop):
    def __init__(self, parent=None):
        super().__init__(parent=parent)

    @property
    def family(self) -> str:
        """The font family"""
        return self.app.family

    @property
    def name(self):
        """The name of the font."""
        return self.app.name

    @property
    def postScriptName(self):
        """The PostScript name of the font."""
        return self.app.postScriptName

    @property
    def style(self):
        """The font style."""
        return self.app.style



"""This object provides an array-style mechanism for storing data.

It can be used for low-level access info Photoshop.


"""
# Import local modules
from photoshop.api._core import Photoshop


class ActionList(Photoshop):
    object_name = "ActionList"

    def __init__(self, parent=None):
        super().__init__(parent=parent)

    @property
    def count(self):
        return self.app.count

    def getBoolean(self, index):
        return self.app.getBoolean(index)

    def getClass(self, index):
        return self.app.getClass(index)

    def getData(self, index):
        return self.app.getData(index)

    def getDouble(self, index):
        return self.app.getDouble(index)

    def getEnumerationType(self, index):
        return self.app.getEnumerationType(index)

    def getEnumerationValue(self, index):
        return self.app.getEnumerationValue(index)

    def getInteger(self, index):
        return self.app.getInteger(index)

    def getLargeInteger(self, index):
        return self.app.getLargeInteger(index)

    def getList(self, index):
        return self.app.getList(index)

    def getObjectType(self, index):
        return self.app.getObjectType(index)



# Import third-party modules
from comtypes import COMError


class PhotoshopPythonAPIError(Exception):
    pass


class PhotoshopPythonAPICOMError(COMError):
    pass


__all__ = ["PhotoshopPythonAPIError", "PhotoshopPythonAPICOMError"]



# Import local modules
from photoshop.api._artlayer import ArtLayer
from photoshop.api._core import Photoshop


# pylint: disable=too-many-public-methods
class ArtLayers(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def _layers(self):
        return list(self.app)

    def __len__(self):
        return self.length

    def __iter__(self):
        for layer in self.app:
            yield layer

    def __getitem__(self, item):
        return self.app[item]

    @property
    def length(self):
        return len(self._layers)

    @property
    def parent(self):
        return self.app.parent

    @property
    def typename(self):
        return self.app.typename

    def add(self):
        """Adds an element."""
        return ArtLayer(self.app.add())

    def getByName(self, name):
        return self.app.getByName(name)

    def removeAll(self):
        return self.app.removeAll()



"""The active containment object for layers and all other objects.

The basic canvas for the file.

- Access the object for the currently active document through
  `Application.activeDocument.`
- You can access other documents or iterate through all open documents using
  in the `Application.documents` collection. You can access individual
  documents in the list by index, or use Documents.getByName() to retrieve
  them by name.
- Create documents programmatically using the Documents.add() method.


"""

# Import built-in modules
from pathlib import Path

# Import third-party modules
from comtypes import COMError

# Import local modules
from photoshop.api._artlayer import ArtLayer
from photoshop.api._artlayers import ArtLayers
from photoshop.api._channels import Channels
from photoshop.api._core import Photoshop
from photoshop.api._documentinfo import DocumentInfo
from photoshop.api._layerComps import LayerComps
from photoshop.api._layerSet import LayerSet
from photoshop.api._layerSets import LayerSets
from photoshop.api._layers import Layers
from photoshop.api._selection import Selection
from photoshop.api.enumerations import ExtensionType
from photoshop.api.enumerations import SaveOptions


# pylint: disable=too-many-public-methods
class Document(Photoshop):
    object_name = "Application"

    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def artLayers(self):
        return ArtLayers(self.app.artLayers)

    @property
    def activeLayer(self):
        """The selected layer."""
        type_ = self.eval_javascript("app.activeDocument.activeLayer.typename")
        mappings = {"LayerSet": LayerSet, "ArtLayer": ArtLayer}
        func = mappings[type_]
        return func(self.app.activeLayer)

    @activeLayer.setter
    def activeLayer(self, layer):
        """Sets the select layer as active layer.

        Args:
            layer (._artlayer.ArtLayer or
                   ._layerSet.LayerSet): The artLayer.

        """
        self.app.activeLayer = layer

    @property
    def activeChannels(self):
        """The selected channels."""
        return self.app.activeChannels

    @activeChannels.setter
    def activeChannels(self, channels):
        self.app.activeChannels = channels

    @property
    def activeHistoryBrushSource(self):
        """The history state to use with the history brush."""
        return self.app.activeHistoryBrushSource

    @property
    def activeHistoryState(self):
        """The current history state for this document."""
        return self.app.activeHistoryState

    @property
    def backgroundLayer(self):
        """The background layer for the Document."""
        return self.app.backgroundLayer

    @property
    def bitsPerChannel(self):
        """The number of bits per channel."""
        return self.app.bitsPerChannel

    @bitsPerChannel.setter
    def bitsPerChannel(self, value):
        self.app.bitsPerChannel = value

    @property
    def channels(self):
        return Channels(self.app.channels)

    @property
    def colorProfileName(self):
        """The name of the color profile. Valid only when no value is specified
        for color profile kind (to indicate a custom color profile)."""
        return self.app.colorProfileName

    @colorProfileName.setter
    def colorProfileName(self, name):
        self.app.colorProfileName = name

    @property
    def colorProfileType(self):
        """The type of color model that defines the working space of the
        Document."""
        return self.app.colorProfileType

    @colorProfileType.setter
    def colorProfileType(self, profile_type):
        self.app.colorProfileType = profile_type

    @property
    def colorSamplers(self):
        """The current color samplers associated with the Document."""
        return self.app.colorSamplers

    @property
    def componentChannels(self):
        """The color component channels for this Document."""
        return self.app.componentChannels

    @property
    def countItems(self):
        """The current count items in the Document."""
        return self.app.countItems

    @property
    def fullName(self):
        """The full path name of the Document."""
        try:
            return Path(self.app.fullName)
        except COMError:
            self.eval_javascript(
                'alert ("Please save your Document first!",' '"{}")'.format(self.name),
            )

    @property
    def height(self):
        """The height of the Document."""
        return self.app.Height

    @property
    def histogram(self):
        """A histogram showing the number of pixels at each color intensity
        level for the composite channel."""
        return self.app.Histogram

    @property
    def history_states(self):
        """The history states collection in this Document."""
        return self.app.HistoryStates

    @property
    def id(self):
        """The unique ID of this Document."""
        return self.app.Id

    @property
    def info(self):
        """Metadata about the Document."""
        return DocumentInfo(self.app.info)

    @property
    def layerComps(self):
        """The layer comps collection in this Document."""
        return LayerComps(self.app.layerComps)

    @property
    def layers(self):
        """The layers collection in the Document."""
        return Layers(self.app.Layers)

    @property
    def layerSets(self):
        """The layer sets collection in the Document."""
        return LayerSets(self.app.layerSets)

    @property
    def managed(self):
        """If true, the Document is a workgroup Document."""
        return self.app.Managed

    @property
    def measurement_scale(self):
        """The measurement scale of the Document."""
        return self.app.MeasurementScale

    @property
    def mode(self):
        """The color profile."""
        return self.app.Mode

    @property
    def name(self) -> str:
        """The Document name."""
        return self.app.name

    @property
    def parent(self):
        """The object's container."""
        return self.app.Parent

    @property
    def path(self):
        """The path to the Document."""
        try:
            return Path(self.app.path)
        except COMError:
            self.eval_javascript(
                'alert ("Please save your Document first!",' '"{}")'.format(self.name),
            )

    @path.setter
    def path(self, path):
        self.app.fullName = path

    @property
    def pathItems(self):
        return self.app.pathItems

    @property
    def pixelAspectRatio(self):
        """The (custom) pixel aspect ratio of the Document.

        Range: 0.100 to 10.000.

        """
        return self.app.pixelAspectRatio

    @property
    def printSettings(self):
        """Document print settings."""
        return self.app.printSettings

    @property
    def quickMaskMode(self):
        """If true, the document is in Quick Mask mode."""
        return self.app.quickMaskMode

    @property
    def saved(self):
        """If true, the Document been saved since the last change."""
        return self.app.Saved

    @property
    def resolution(self):
        """The resolution of the Document (in pixels per inch)"""
        return self.app.resolution

    @property
    def selection(self):
        """The selected area of the Document."""
        return Selection(self.app.selection)

    @property
    def typename(self):
        """The class name of the object."""
        return self.app.typename

    @property
    def cloudDocument(self):
        """This document is in the cloud."""
        return self.app.cloudDocument

    @property
    def cloudWorkAreaDirectory(self):
        """Local directory for this cloud document."""
        return self.app.cloudWorkAreaDirectory

    @property
    def width(self):
        return self.app.Width

    @property
    def xmpMetadata(self):
        """The XMP properties of the Document. The Camera RAW settings are
        stored here."""
        return self.app.xmpMetadata

    # Methods
    def autoCount(self, *args, **kwargs):
        """Counts the objects in the Document."""
        return self.app.autoCount(*args, **kwargs)

    def changeMode(self, *args, **kwargs):
        """Changes the mode of the Document."""
        return self.app.changeMode(*args, **kwargs)

    def close(self, saving=SaveOptions.DoNotSaveChanges):
        return self.app.close(saving)

    def convertProfile(self):
        return self.app.convertProfile()

    def flatten(self):
        """Flattens all layers."""
        return self.app.Flatten()

    def mergeVisibleLayers(self):
        """Flattens all visible layers in the Document."""
        return self.app.mergeVisibleLayers()

    def crop(self, bounds, angle=None, width=None, height=None):
        """Crops the document.

        Args:
            bounds (list of int): Four coordinates for the region remaining
                after cropping.
            angle (float, optional): The angle of cropping bounds.
            width (int, optional): The width of the resulting document.
            height (int, optional): The height of the resulting document.

        """
        return self.app.crop(bounds, angle, width, height)

    def exportDocument(self, file_path, exportAs=None, options=None):
        """Exports the Document."""
        return self.app.exportDocument(file_path, exportAs, options)

    def duplicate(self, name=None, merge_layers_only=False):
        return Document(self.app.duplicate(name, merge_layers_only))

    def paste(self):
        """Pastes contents of the clipboard into the Document."""
        return self.app.paste()

    def print(self):
        """Prints the document."""
        return self.app.print()

    def printOneCopy(self):
        self.app.printOneCopy()

    def rasterizeAllLayers(self):
        return self.app.rasterizeAllLayers()

    def recordMeasurements(self, source, dataPoints):
        """Records the measurements of document."""
        self.app.recordMeasurements(source, dataPoints)

    def reveal_all(self):
        """Expands the Document to show clipped sections."""
        return self.app.revealAll()

    def save(self):
        """Saves the Document."""
        return self.app.save()

    def saveAs(
        self, file_path, options, asCopy=True, extensionType=ExtensionType.Lowercase
    ):
        """Saves the documents with the specified save options.

        Args:
            file_path (str): Absolute path of psd file.
            options (.JPEGSaveOptions): Save options.
            asCopy (bool):
        """
        return self.app.saveAs(file_path, options, asCopy, extensionType)

    def splitChannels(self):
        """Splits the channels of the document."""
        self.app.splitChannels()

    def suspendHistory(self, historyString, javaScriptString):
        """Provides a single history state for the entire script.

        Allows a single undo for all actions taken in the script.

        """
        self.eval_javascript(
            f"app.activeDocument.suspendHistory('{historyString}',"
            f" '{javaScriptString}')"
        )

    def trap(self, width):
        """
        Applies trapping to a CMYK document.
        Valid only when ‘mode’ = CMYK.

        """
        self.app.trap(width)

    def trim(self, trim_type, top=True, left=True, bottom=True, right=True):
        """Trims the transparent area around the image on the specified
        sides of the canvas.

        Args:
            trim_type (TrimType): The color or type of pixels to base the trim
                on.
                .e.g:
                    - TrimType.BottomRightPixel
                    - TrimType.TopLeftPixel
                    - TrimType.TransparentPixels
            top (bool, optional): If true, trims away the top of the document.
            left (bool, optional): If true, trims away the left of the
                document.
            bottom (bool, optional): If true, trims away the bottom of the
                document.
            right (bool, optional): 	If true, trims away the right of the
                document.

        """
        return self.app.trim(trim_type, top, left, bottom, right)

    def resizeImage(self, width, height, resolution=72, automatic=8):
        """Changes the size of the image.

        Args:
            width: The desired width of the image.
            height: The desired height of the image.
            resolution: The resolution (in pixels per inch)

        """
        return self.app.resizeImage(width, height, resolution, automatic)



# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api._layerComp import LayerComp
from photoshop.api.errors import PhotoshopPythonAPIError


class LayerComps(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    def __len__(self):
        return self.length

    @property
    def length(self):
        return len(self._layers)

    @property
    def _layers(self):
        return list(self.app)

    @property
    def parent(self):
        return self.app.parent

    @property
    def typename(self):
        return self.app.typename

    def add(
        self,
        name,
        comment="No Comment.",
        appearance=True,
        position=True,
        visibility=True,
        childLayerCompStat=False,
    ):
        return LayerComp(
            self.app.add(
                name, comment, appearance, position, visibility, childLayerCompStat
            )
        )

    def getByName(self, name):
        for layer in self._layers:
            if layer.name == name:
                return LayerComp(layer)
        raise PhotoshopPythonAPIError(f'Could not find a layer named "{name}"')

    def removeAll(self):
        self.app.removeAll()

    def __iter__(self):
        for layer in self._layers:
            yield LayerComp(layer)



# Import local modules
from photoshop.api._core import Photoshop


class LayerComp(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    def __len__(self):
        return self.length

    @property
    def appearance(self):
        return self.app.appearance

    @appearance.setter
    def appearance(self, value):
        self.app.appearance = value

    @property
    def childLayerCompState(self):
        return self.app.childLayerCompState

    @childLayerCompState.setter
    def childLayerCompState(self, value):
        self.app.childLayerCompState = value

    @property
    def comment(self):
        return self.app.comment

    @comment.setter
    def comment(self, text):
        self.app.comment = text

    @property
    def name(self):
        return self.app.name

    @name.setter
    def name(self, text):
        self.app.name = text

    @property
    def parent(self):
        return self.app.parent

    @property
    def position(self):
        return self.app.position

    @position.setter
    def position(self, value):
        self.app.position = value

    @property
    def selected(self):
        """True if the layer comp is currently selected."""
        return self.app.selected

    @selected.setter
    def selected(self, value):
        self.app.selected = value

    @property
    def typename(self):
        return self.app.typename

    @property
    def visibility(self):
        """True to use layer visibility settings."""
        return self.app.visibility

    @visibility.setter
    def visibility(self, value):
        self.app.visibility = value

    def apply(self):
        """Applies the layer comp to the document."""
        self.app.apply()

    def recapture(self):
        """Recaptures the current layer state(s) for this layer comp."""
        self.app.recapture()

    def remove(self):
        """Deletes the layerComp object."""
        self.app.remove()

    def resetfromComp(self):
        """Resets the layer comp state to thedocument state."""
        self.app.resetfromComp()



"""This object provides information about what the action is refering to.

For example, when referring to the name of something you might use keyName.
The reference would also need to know what name you are referring to.
In this case you could use classDocument for the name of the document or
classLayer for the name of the layer.
It can be used for low-level access into Contains data associated
with an ActionDescriptor.

"""
# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api.enumerations import ReferenceFormType


class ActionReference(Photoshop):
    object_name = "ActionReference"

    def __init__(self, parent=None):
        super().__init__(parent=parent)

    def getContainer(self):
        return self.app.getContainer()

    def getDesiredClass(self):
        return self.app.getDesiredClass()

    def getEnumeratedType(self) -> int:
        return self.app.getEnumeratedType()

    def getEnumeratedValue(self) -> int:
        return self.app.getEnumeratedValue()

    def getForm(self) -> ReferenceFormType:
        """Gets the form of this action reference."""
        return ReferenceFormType(self.app.getForm())

    def getIdentifier(self) -> int:
        """Gets the identifier value for a reference whose form is
        identifier."""
        return self.app.getIdentifier()

    def getIndex(self) -> int:
        """Gets the index value for a reference in a list or array,"""
        return self.app.getIndex()

    def putName(self, key, value):
        return self.app.putName(key, value)

    def putClass(self, value):
        return self.app.putClass(value)

    def putEnumerated(self, desired_class, enum_type, value):
        """Puts an enumeration type and ID into a reference along with the
        desired class for the reference."""
        return self.app.putEnumerated(desired_class, enum_type, value)

    def putIdentifier(self, desired_class, value):
        return self.app.putIdentifier(desired_class, value)

    def putIndex(self, desired_class, value):
        return self.app.putIndex(desired_class, value)

    def putOffset(self, desired_class, value):
        return self.app.putOffset(desired_class, value)

    def putProperty(self, desired_class, value):
        return self.app.putProperty(desired_class, value)



"""
The collection of Notifier objects in the document. Access through the
Application.notifiers collection property.

For example:
var notRef = app.notifiers.add("OnClickGoButton", eventFile)
Notifiers must be enabled using the Application.notifiersEnabled property.

"""

# Import built-in modules
from typing import Any
from typing import Optional

# Import local modules
from photoshop.api._core import Photoshop
from photoshop.api._notifier import Notifier


class Notifiers(Photoshop):
    def __init__(self, parent: Optional[Any] = None):
        super().__init__(parent=parent)

    @property
    def _notifiers(self) -> list:
        return [n for n in self.app]

    def __len__(self):
        return self.length

    def __iter__(self):
        for app in self.app:
            yield app

    def __getitem__(self, item):
        return self._notifiers[item]

    @property
    def length(self):
        return len(self._notifiers)

    def add(
        self, event, event_file: Optional[Any] = None, event_class: Optional[Any] = None
    ) -> Notifier:
        self.parent.notifiersEnabled = True
        return Notifier(self.app.add(event, event_file, event_class))

    def removeAll(self):
        self.app.removeAll()
        self.parent.notifiersEnabled = False



"""constants type of enum for Photoshop."""
# Import built-in modules
from enum import IntEnum


class LensType(IntEnum):
    MoviePrime = 5
    Prime105 = 3
    Prime35 = 2
    ZoomLens = 1


class AdjustmentReference(IntEnum):
    Absolute = 2
    Relative = 1


class AnchorPosition(IntEnum):
    BottomCenter = 8
    BottomLeft = 7
    BottomRight = 9
    MiddleCenter = 5
    MiddleLeft = 4
    MiddleRight = 6
    TopCenter = 2
    TopLeft = 1
    TopRight = 3


class AntiAlias(IntEnum):
    Crisp = 3
    NoAntialias = 1
    Sharp = 2
    Smooth = 5
    Strong = 4


class AutoKernType(IntEnum):
    Manual = 1
    Metrics = 2
    Optical = 3


class BMPDepthType(IntEnum):
    BMP16Bits = 16
    BMP1Bit = 1
    BMP24Bits = 24
    BMP32Bits = 32
    BMP4Bits = 4
    BMP8Bits = 8
    BMP_A1R5G5B5 = 61
    BMP_A4R4G4B4 = 64
    BMP_A8R8G8B8 = 67
    BMP_R5G6B5 = 62
    BMP_R8G8B8 = 65
    BMP_X1R5G5B5 = 60
    BMP_X4R4G4B4 = 63
    BMP_X8R8G8B8 = 66


class BatchDestinationType(IntEnum):
    Folder = 3
    NoDestination = 1
    SaveAndClose = 2


class BitmapConversionType(IntEnum):
    CustomPattern = 5
    DiffusionDither = 3
    HalfThreshold = 1
    HalftoneScreen = 4
    PatternDither = 2


class BitmapHalfToneType(IntEnum):
    HalftoneCross = 6
    HalftoneDiamond = 2
    HalftoneEllie = 3
    HalftoneLine = 4
    HalftoneRound = 1
    HalftoneSquare = 5


class BitsPerChannelType(IntEnum):
    Document16Bits = 16
    Document1Bit = 1
    Document32Bits = 32
    Document8Bits = 8


class BlendMode(IntEnum):
    ColorBlend = 22
    ColorBurn = 6
    ColorDodge = 10
    Darken = 4
    DarkerColor = 28
    Difference = 18
    Dissolve = 3
    Divide = 30
    Exclusion = 19
    HardLight = 14
    HardMix = 26
    Hue = 20
    Lighten = 8
    LighterColor = 27
    LinearBurn = 7
    LinearDodge = 11
    LinearLight = 16
    Luminosity = 23
    Multiply = 5
    NormalBlend = 2
    Overlay = 12
    PassThrough = 1
    PinLight = 17
    SaturationBlend = 21
    Screen = 9
    SoftLight = 13
    Subtract = 29
    VividLight = 15


class ByteOrderType(IntEnum):
    IBMByteOrder = 1
    MacOSByteOrder = 2


class CameraRAWSettingsType(IntEnum):
    CameraDefault = 0
    CustomSettings = 2
    SelectedImage = 1


class CameraRAWSize(IntEnum):
    ExtraLargeCameraRAW = 4
    LargeCameraRAW = 3
    MaximumCameraRAW = 5
    MediumCameraRAW = 2
    MinimumCameraRAW = 0
    SmallCameraRAW = 1


class Case(IntEnum):
    AllCa = 2
    NormalCase = 1
    SmallCa = 3


class ChangeMode(IntEnum):
    ConvertToBitmap = 5
    ConvertToCMYK = 3
    ConvertToGrayscale = 1
    ConvertToIndexedColor = 6
    ConvertToLab = 4
    ConvertToMultiChannel = 7
    ConvertToRGB = 2


class ChannelType(IntEnum):
    ComponentChannel = 1
    MaskedAreaAlphaChannel = 2
    SelectedAreaAlphaChannel = 3
    SpotColorChannel = 4


class ColorBlendMode(IntEnum):
    ColorBlendMode = 22
    BehindBlend = 24
    ClearBlend = 25
    ColorBurnBlend = 6
    ColorDodgeBlend = 10
    DarkenBlend = 4
    DarkerColorBlend = 28
    DifferenceBlend = 18
    DissolveBlend = 3
    DivideBlend = 30
    ExclusionBlend = 19
    HardLightBlend = 14
    HardMixBlend = 26
    HueBlend = 20
    LightenBlend = 8
    LighterColorBlend = 27
    LinearBurnBlend = 7
    LinearDodgeBlend = 11
    LinearLightBlend = 16
    LuminosityBlend = 23
    MultiplyBlend = 5
    NormalBlendColor = 2
    OverlayBlend = 12
    PinLightBlend = 17
    SaturationBlendColor = 21
    ScreenBlend = 9
    SoftLightBlend = 13
    SubtractBlend = 29
    VividLightBlend = 15


class ColorModel(IntEnum):
    CMYKModel = 3
    GrayscaleModel = 1
    HSBModel = 5
    LabModel = 4
    NoModel = 50
    RGBModel = 2


class ColorPicker(IntEnum):
    AdobeColorPicker = 1
    AppleColorPicker = 2
    PlugInColorPicker = 4
    WindowsColorPicker = 3


class ColorProfileType(IntEnum):
    Custom = 3
    No = 1
    Working = 2


class ColorReductionType(IntEnum):
    Adaptive = 2
    BlackWhiteReduction = 5
    CustomReduction = 4
    MacintoshColors = 7
    PerceptualReduction = 0
    Restrictive = 3
    SFWGrayscale = 6
    Selective = 1
    WindowsColors = 8


class ColorSpaceType(IntEnum):
    AdobeRGB = 0
    ColorMatchRGB = 1
    ProPhotoRGB = 2
    SRGB = 3


class CopyrightedType(IntEnum):
    CopyrightedWork = 1
    PublicDomain = 2
    Unmarked = 3


class CreateFields(IntEnum):
    Duplication = 1
    Interpolation = 2


class CropToType(IntEnum):
    ArtBox = 5
    BleedBox = 3
    BoundingBox = 0
    CropBox = 2
    MediaBox = 1
    TrimBox = 4


class DCSType(IntEnum):
    ColorComposite = 3
    GrayscaleComposite = 2
    NoComposite = 1


class DepthMaource(IntEnum):
    ImageHighlight = 4
    LayerMask = 3
    NoSource = 1
    TransparencyChannel = 2


class DescValueType(IntEnum):
    AliasType = 11
    BooleanType = 5
    ClassType = 10
    DoubleType = 2
    EnumeratedType = 8
    IntegerType = 1
    LargeIntegerType = 13
    ListType = 6
    ObjectType = 7
    RawType = 12
    ReferenceType = 9
    StringType = 4
    UnitDoubleType = 3


class DialogModes(IntEnum):
    DisplayAllDialogs = 1
    DisplayErrorDialogs = 2
    DisplayNoDialogs = 3


class Direction(IntEnum):
    Horizontal = 1
    Vertical = 2


class DisplacementMapType(IntEnum):
    StretchToFit = 1
    Tile = 2


class DitherType(IntEnum):
    Diffusion = 2
    NoDither = 1
    Noise = 4
    Pattern = 3


class DocumentFill(IntEnum):
    BackgroundColor = 2
    Transparent = 3
    White = 1


class DocumentMode(IntEnum):
    Bitmap = 5
    CMYK = 3
    Duotone = 8
    Grayscale = 1
    IndexedColor = 6
    Lab = 4
    MultiChannel = 7
    RGB = 2


class EditLogItemsType(IntEnum):
    Concise = 2
    Detailed = 3
    SessionOnly = 1


class ElementPlacement(IntEnum):
    PlaceAfter = 4
    PlaceAtBeginning = 1
    PlaceAtEnd = 2
    PlaceBefore = 3
    PlaceInside = 0


class EliminateFields(IntEnum):
    EvenFields = 2
    OddFields = 1


class ExportType(IntEnum):
    IllustratorPaths = 1
    SaveForWeb = 2


class ExtensionType(IntEnum):
    Lowercase = 2
    Uppercase = 3


class FileNamingType(IntEnum):
    Ddmm = 16
    Ddmmyy = 15
    DocumentNameLower = 2
    DocumentNameMixed = 1
    DocumentNameUpper = 3
    ExtensionLower = 17
    ExtensionUpper = 18
    Mmdd = 11
    Mmddyy = 10
    SerialLetterLower = 8
    SerialLetterUpper = 9
    SerialNumber1 = 4
    SerialNumber2 = 5
    SerialNumber3 = 6
    SerialNumber4 = 7
    Yyddmm = 14
    Yymmdd = 13
    Yyyymmdd = 12


class FontPreviewType(IntEnum):
    FontPreviewExtraLarge = 4
    FontPreviewHuge = 5
    FontPreviewLarge = 3
    FontPreviewMedium = 2
    FontPreviewNone = 0
    FontPreviewSmall = 1


class ForcedColors(IntEnum):
    BlackWhite = 2
    NoForced = 1
    Primaries = 3
    Web = 4


class FormatOptionsType(IntEnum):
    OptimizedBaseline = 2
    Progressive = 3
    StandardBaseline = 1


class GalleryConstrainType(IntEnum):
    ConstrainBoth = 3
    ConstrainHeight = 2
    ConstrainWidth = 1


class GalleryFontType(IntEnum):
    Arial = 1
    CourierNew = 2
    Helvetica = 3
    TimesNewRoman = 4


class GallerySecurityTextColorType(IntEnum):
    BlackText = 1
    CustomText = 3
    WhiteText = 2


class GallerySecurityTextPositionType(IntEnum):
    Centered = 1
    LowerLeft = 3
    LowerRight = 5
    UpperLeft = 2
    UpperRight = 4


class GallerySecurityTextRotateType(IntEnum):
    Clockwise45 = 2
    Clockwise90 = 3
    CounterClockwise45 = 4
    CounterClockwise90 = 5
    Zero = 1


class GallerySecurityType(IntEnum):
    Caption = 5
    Copyright = 4
    Credit = 6
    CustomSecurityText = 2
    Filename = 3
    NoSecurity = 1
    Title = 7


class GalleryThumbSizeType(IntEnum):
    CustomThumbnail = 4
    Large = 3
    Medium = 2
    Small = 1


class Geometry(IntEnum):
    Heptagon = 4
    Hexagon = 2
    Octagon = 5
    Pentagon = 1
    SquareGeometry = 3
    Triangle = 0


class GridLineStyle(IntEnum):
    GridDashedLine = 2
    GridDottedLine = 3
    GridSolidLine = 1


class GridSize(IntEnum):
    LargeGrid = 4
    MediumGrid = 3
    NoGrid = 1
    SmallGrid = 2


class GuideLineStyle(IntEnum):
    GuideDashedLine = 2
    GuideSolidLine = 1


class IllustratorPathType(IntEnum):
    AllPaths = 2
    DocumentBounds = 1
    NamedPath = 3


class Intent(IntEnum):
    AbsoluteColorimetric = 4
    Perceptual = 1
    RelativeColorimetric = 3
    Saturation = 2


class JavaScriptExecutionMode(IntEnum):
    BeforeRunning = 3
    DebuggerOnError = 2
    NeverShowDebugger = 1


class Justification(IntEnum):
    Center = 2
    CenterJustified = 5
    FullyJustified = 7
    Left = 1
    LeftJustified = 4
    Right = 3
    RightJustified = 6


class Language(IntEnum):
    BrazillianPortuguese = 13
    CanadianFrench = 4
    Danish = 17
    Dutch = 16
    EnglishUK = 2
    EnglishUSA = 1
    Finnish = 5
    French = 3
    German = 6
    Italian = 9
    Norwegian = 10
    NynorskNorwegian = 11
    OldGerman = 7
    Portuguese = 12
    Spanish = 14
    Swedish = 15
    SwissGerman = 8


class LayerCompressionType(IntEnum):
    RLELayerCompression = 1
    ZIPLayerCompression = 2


class LayerKind(IntEnum):
    BlackAndWhiteLayer = 22
    BrightnessContrastLayer = 9
    ChannelMixerLayer = 12
    ColorBalanceLayer = 8
    ColorLookup = 24
    CurvesLayer = 7
    ExposureLayer = 19
    GradientFillLayer = 4
    GradientMapLayer = 13
    HueSaturationLayer = 10
    InversionLayer = 14
    Layer3D = 20
    LevelsLayer = 6
    NormalLayer = 1
    PatternFillLayer = 5
    PhotoFilterLayer = 18
    PosterizeLayer = 16
    SelectiveColorLayer = 11
    SmartObjectLayer = 17
    SolidFillLayer = 3
    TextLayer = 2
    ThresholdLayer = 15
    Vibrance = 23
    VideoLayer = 21


class LayerType(IntEnum):
    ArtLayer = 1
    LayerSet = 2


class MagnificationType(IntEnum):
    ActualSize = 0
    FitPage = 1


class MatteType(IntEnum):
    BackgroundColorMatte = 3
    BlackMatte = 5
    ForegroundColorMatte = 2
    NetscapeGrayMatte = 7
    NoMatte = 1
    SemiGray = 6
    WhiteMatte = 4


class MeasurementRange(IntEnum):
    ActiveMeasurements = 2
    AllMeasurements = 1


class MeasurementSource(IntEnum):
    MeasureCountTool = 2
    MeasureRulerTool = 3
    MeasureSelection = 1


class NewDocumentMode(IntEnum):
    NewBitmap = 5
    NewCMYK = 3
    NewGray = 1
    NewLab = 4
    NewRGB = 2


class NoiseDistribution(IntEnum):
    GaussianNoise = 2
    UniformNoise = 1


class OffsetUndefinedAreas(IntEnum):
    OffsetRepeatEdgePixels = 3
    OffsetSetToLayerFill = 1
    OffsetWrapAround = 2


class OpenDocumentMode(IntEnum):
    OpenCMYK = 3
    OpenGray = 1
    OpenLab = 4
    OpenRGB = 2


class OpenDocumentType(IntEnum):
    AliasPIXOpen = 25
    BMPOpen = 2
    CameraRAWOpen = 32
    CompuServeGIFOpen = 3
    DICOMOpen = 33
    EOpen = 22
    EPICTPreviewOpen = 23
    ETIFFPreviewOpen = 24
    ElectricImageOpen = 26
    FilmstripOpen = 5
    JPEGOpen = 6
    PCXOpen = 7
    PDFOpen = 21
    PICTFileFormatOpen = 10
    PICTResourceFormatOpen = 11
    PNGOpen = 13
    PhotoCDOpen = 9
    PhotoshopDCS_1Open = 18
    PhotoshopDCS_2Open = 19
    PhotoshopEOpen = 4
    PhotoshopOpen = 1
    PhotoshopPDFOpen = 8
    PixarOpen = 12
    PortableBitmapOpen = 27
    RawOpen = 14
    SGIRGBOpen = 29
    ScitexCTOpen = 15
    SoftImageOpen = 30
    TIFFOpen = 17
    TargaOpen = 16
    WavefrontRLAOpen = 28
    WirelessBitmapOpen = 31


class OperatingSystem(IntEnum):
    OS2 = 1
    Windows = 2


class Orientation(IntEnum):
    Landscape = 1
    Portrait = 2


class OtherPaintingCursors(IntEnum):
    PreciseOther = 2
    StandardOther = 1


class PDFCompatibilityType(IntEnum):
    PDF13 = 1
    PDF14 = 2
    PDF15 = 3
    PDF16 = 4
    PDF17 = 5


class PDFEncodingType(IntEnum):
    PDFJPEG = 2
    PDFJPEG2000HIGH = 9
    PDFJPEG2000LOSSLESS = 14
    PDFJPEG2000LOW = 13
    PDFJPEG2000MED = 11
    PDFJPEG2000MEDHIGH = 10
    PDFJPEG2000MEDLOW = 12
    PDFJPEGHIGH = 4
    PDFJPEGLOW = 8
    PDFJPEGMED = 6
    PDFJPEGMEDHIGH = 5
    PDFJPEGMEDLOW = 7
    PDFNone = 0
    PDFZip = 1
    PDFZip4Bit = 3


class PDFResampleType(IntEnum):
    NoResample = 0
    PDFAverage = 1
    PDFBicubic = 3
    PDFSubSample = 2


class PDFStandardType(IntEnum):
    NoStandard = 0
    PDFX1A2001 = 1
    PDFX1A2003 = 2
    PDFX32002 = 3
    PDFX32003 = 4
    PDFX42008 = 5


class PICTBitsPerPixel(IntEnum):
    PICT16Bits = 16
    PICT2Bits = 2
    PICT32Bits = 32
    PICT4Bits = 4
    PICT8Bits = 8


class PICTCompression(IntEnum):
    JPEGHighPICT = 5
    JPEGLowPICT = 2
    JPEGMaximumPICT = 6
    JPEGMediumPICT = 4
    NoPICTCompression = 1


class PaintingCursors(IntEnum):
    BrushSize = 3
    Precise = 2
    Standard = 1


class PaletteType(IntEnum):
    Exact = 1
    LocalAdaptive = 8
    LocalPerceptual = 6
    LocalSelective = 7
    MacOSPalette = 2
    MasterAdaptive = 11
    MasterPerceptual = 9
    MasterSelective = 10
    PreviousPalette = 12
    Uniform = 5
    WebPalette = 4
    WindowsPalette = 3


class PathKind(IntEnum):
    ClippingPath = 2
    NormalPath = 1
    TextMask = 5
    VectorMask = 4
    WorkPath = 3


class PhotoCDColorSpace(IntEnum):
    Lab16 = 4
    Lab8 = 3
    RGB16 = 2
    RGB8 = 1


class PhotoCDSize(IntEnum):
    ExtraLargePhotoCD = 5
    LargePhotoCD = 4
    MaximumPhotoCD = 6
    MediumPhotoCD = 3
    MinimumPhotoCD = 1
    SmallPhotoCD = 2


class PicturePackageTextType(IntEnum):
    CaptionText = 5
    CopyrightText = 4
    CreditText = 6
    FilenameText = 3
    NoText = 1
    OriginText = 7
    UserText = 2


class PointKind(IntEnum):
    CornerPoint = 2
    SmoothPoint = 1


class PointType(IntEnum):
    PostScriptPoints = 1
    TraditionalPoints = 2


class PolarConversionType(IntEnum):
    PolarToRectangular = 2
    RectangularToPolar = 1


class PreviewType(IntEnum):
    EightBitTIFF = 3
    MonochromeTIFF = 2
    NoPreview = 1


class PurgeTarget(IntEnum):
    AllCaches = 4
    ClipboardCache = 3
    HistoryCaches = 2
    UndoCaches = 1


class QueryStateType(IntEnum):
    Always = 1
    Ask = 2
    Never = 3


class RadialBlurMethod(IntEnum):
    Spin = 1
    Zoom = 2


class RadialBlurBest(IntEnum):
    RadialBlurBest = 3
    RadialBlurDraft = 1
    RadialBlurGood = 2


class RasterizeType(IntEnum):
    EntireLayer = 5
    FillContent = 3
    LayerClippingPath = 4
    LinkedLayers = 6
    Shape = 2
    TextContents = 1


class ReferenceFormType(IntEnum):
    ReferenceClassType = 7
    ReferenceEnumeratedType = 5
    ReferenceIdentifierType = 3
    ReferenceIndexType = 2
    ReferenceNameType = 1
    ReferenceOffsetType = 4
    ReferencePropertyType = 6


class ResampleMethod(IntEnum):
    Automatic = 8
    Bicubic = 4
    BicubicAutomatic = 7
    BicubicSharper = 5
    BicubicSmoother = 6
    Bilinear = 3
    NearestNeighbor = 2
    NoResampling = 1
    PreserveDetails = 9


class ResetTarget(IntEnum):
    AllTools = 2
    AllWarnings = 1
    Everything = 3


class RippleSize(IntEnum):
    LargeRipple = 3
    MediumRipple = 2
    SmallRipple = 1


class SaveBehavior(IntEnum):
    AlwaysSave = 2
    AskWhenSaving = 3
    NeverSave = 1


class SaveDocumentType(IntEnum):
    AliasPIXSave = 25
    BMave = 2
    CompuServeGIFSave = 3
    ElectricImageSave = 26
    JPEGSave = 6
    PCXSave = 7
    PICTFileFormatSave = 10
    PICTResourceFormatSave = 11
    PNGSave = 13
    PhotoshopDCS_1Save = 18
    PhotoshopDCS_2Save = 19
    PhotoshopESave = 4
    PhotoshopPDFSave = 8
    Photoshoave = 1
    PixarSave = 12
    PortableBitmaave = 27
    RawSave = 14
    SGIRGBSave = 29
    ScitexCTSave = 15
    SoftImageSave = 30
    TIFFSave = 17
    TargaSave = 16
    WavefrontRLASave = 28
    WirelessBitmaave = 31


class SaveEncoding(IntEnum):
    Ascii = 3
    Binary = 1
    JPEGHigh = 5
    JPEGLow = 2
    JPEGMaximum = 6
    JPEGMedium = 4


class SaveLogItemsType(IntEnum):
    LogFile = 2
    LogFileAndMetadata = 3
    Metadata = 1


class SaveOptions(IntEnum):
    DoNotSaveChanges = 2
    PromptToSaveChanges = 3
    SaveChanges = 1


class SelectionType(IntEnum):
    DiminishSelection = 3
    ExtendSelection = 2
    IntersectSelection = 4
    ReplaceSelection = 1


class ShapeOperation(IntEnum):
    ShapeAdd = 1
    ShapeIntersect = 3
    ShapeSubtract = 4
    ShapeXOR = 2


class SmartBlurMode(IntEnum):
    SmartBlurEdgeOnly = 2
    SmartBlurNormal = 1
    SmartBlurOverlayEdge = 3


class SmartBlurQuality(IntEnum):
    SmartBlurHigh = 3
    SmartBlurLow = 1
    SmartBlurMedium = 2


class SourceSpaceType(IntEnum):
    DocumentSpace = 1
    ProofSpace = 2


class SpherizeMode(IntEnum):
    HorizontalSpherize = 2
    NormalSpherize = 1
    VerticalSpherize = 3


class StrikeThruType(IntEnum):
    StrikeBox = 3
    StrikeHeight = 2
    StrikeOff = 1


class StrokeLocation(IntEnum):
    CenterStroke = 2
    InsideStroke = 1
    OutsideStroke = 3


class TargaBitsPerPixels(IntEnum):
    Targa16Bits = 16
    Targa24Bits = 24
    Targa32Bits = 32


class TextComposer(IntEnum):
    AdobeEveryLine = 2
    AdobeSingleLine = 1


class TextType(IntEnum):
    ParagraphText = 2
    PointText = 1


class TextureType(IntEnum):
    BlocksTexture = 1
    CanvasTexture = 2
    FrostedTexture = 3
    TextureFile = 5
    TinyLensTexture = 4


class TiffEncodingType(IntEnum):
    NoTIFFCompression = 1
    TiffJPEG = 3
    TiffLZW = 2
    TiffZIP = 4


class ToolType(IntEnum):
    ArtHistoryBrush = 9
    BackgroundEraser = 4
    Blur = 11
    Brush = 2
    Burn = 14
    CloneStamp = 5
    ColorReplacementTool = 16
    Dodge = 13
    Eraser = 3
    HealingBrush = 7
    HistoryBrush = 8
    PatternStamp = 6
    Pencil = 1
    Sharpen = 12
    Smudge = 10
    Sponge = 15


class TransitionType(IntEnum):
    BlindsHorizontal = 1
    BlindsVertical = 2
    BoxIn = 4
    BoxOut = 5
    DissolveTransition = 3
    GlitterDown = 6
    GlitterRight = 7
    GlitterRightDown = 8
    NoTrasition = 9
    Random = 10
    SplitHorizontalIn = 11
    SplitHorizontalOut = 12
    SplitVerticalIn = 13
    SplitVerticalOut = 14
    WipeDown = 15
    WipeLeft = 16
    WipeRight = 17
    WipeUp = 18


class TrimType(IntEnum):
    BottomRightPixel = 9
    TopLeftPixel = 1
    TransparentPixels = 0


class TypeUnits(IntEnum):
    TypeMM = 4
    TypePixels = 1
    TypePoints = 5


class UndefinedAreas(IntEnum):
    RepeatEdgePixels = 2
    WrapAround = 1


class UnderlineType(IntEnum):
    UnderlineLeft = 3
    UnderlineOff = 1
    UnderlineRight = 2


class Units(IntEnum):
    CM = 3
    Inches = 2
    MM = 4
    Percent = 7
    Picas = 6
    Pixels = 1
    Points = 5


class Urgency(IntEnum):
    Four = 4
    High = 8
    Low = 1
    UrgencyNone = 0
    Normal = 5
    Seven = 7
    Six = 6
    Three = 3
    Two = 2


class Wartyle(IntEnum):
    Arc = 2
    ArcLower = 3
    ArcUpper = 4
    Arch = 5
    Bulge = 6
    Fish = 11
    FishEye = 13
    Flag = 9
    Inflate = 14
    NoWarp = 1
    Rise = 12
    ShellLower = 7
    ShellUpper = 8
    Squeeze = 15
    Twist = 16
    Wave = 10


class WaveType(IntEnum):
    Sine = 1
    Square = 3
    Triangular = 2


class WhiteBalanceType(IntEnum):
    AsShot = 0
    Auto = 1
    Cloudy = 3
    CustomCameraSettings = 8
    Daylight = 2
    Flash = 7
    Fluorescent = 6
    Shade = 4
    Tungsten = 5


class ZigZagType(IntEnum):
    AroundCenter = 1
    OutFromCenter = 2
    PondRipples = 3


__all__ = [
    "LensType",
    "AdjustmentReference",
    "AnchorPosition",
    "AntiAlias",
    "AutoKernType",
    "BMPDepthType",
    "BatchDestinationType",
    "BitmapConversionType",
    "BitmapHalfToneType",
    "BitsPerChannelType",
    "BlendMode",
    "ByteOrderType",
    "CameraRAWSettingsType",
    "CameraRAWSize",
    "Case",
    "ChangeMode",
    "ChannelType",
    "ColorBlendMode",
    "ColorModel",
    "ColorPicker",
    "ColorProfileType",
    "ColorReductionType",
    "ColorSpaceType",
    "CopyrightedType",
    "CreateFields",
    "CropToType",
    "DCSType",
    "DepthMaource",
    "DescValueType",
    "DialogModes",
    "Direction",
    "DisplacementMapType",
    "DitherType",
    "DocumentFill",
    "DocumentMode",
    "EditLogItemsType",
    "ElementPlacement",
    "EliminateFields",
    "ExportType",
    "ExtensionType",
    "FileNamingType",
    "FontPreviewType",
    "ForcedColors",
    "FormatOptionsType",
    "GalleryConstrainType",
    "GalleryFontType",
    "GallerySecurityTextColorType",
    "GallerySecurityTextPositionType",
    "GallerySecurityTextRotateType",
    "GallerySecurityType",
    "GalleryThumbSizeType",
    "Geometry",
    "GridLineStyle",
    "GridSize",
    "GuideLineStyle",
    "IllustratorPathType",
    "Intent",
    "JavaScriptExecutionMode",
    "Justification",
    "Language",
    "LayerCompressionType",
    "LayerKind",
    "LayerType",
    "MagnificationType",
    "MatteType",
    "MeasurementRange",
    "MeasurementSource",
    "NewDocumentMode",
    "NoiseDistribution",
    "OffsetUndefinedAreas",
    "OpenDocumentMode",
    "OpenDocumentType",
    "OperatingSystem",
    "Orientation",
    "OtherPaintingCursors",
    "PDFCompatibilityType",
    "PDFEncodingType",
    "PDFResampleType",
    "PDFStandardType",
    "PICTBitsPerPixel",
    "PICTCompression",
    "PaintingCursors",
    "PaletteType",
    "PathKind",
    "PhotoCDColorSpace",
    "PhotoCDSize",
    "PicturePackageTextType",
    "PointKind",
    "PointType",
    "PolarConversionType",
    "PreviewType",
    "PurgeTarget",
    "QueryStateType",
    "RadialBlurMethod",
    "RadialBlurBest",
    "RasterizeType",
    "ReferenceFormType",
    "ResampleMethod",
    "ResetTarget",
    "RippleSize",
    "SaveBehavior",
    "SaveDocumentType",
    "SaveEncoding",
    "SaveLogItemsType",
    "SaveOptions",
    "SelectionType",
    "ShapeOperation",
    "SmartBlurMode",
    "SmartBlurQuality",
    "SourceSpaceType",
    "SpherizeMode",
    "StrikeThruType",
    "StrokeLocation",
    "TargaBitsPerPixels",
    "TextComposer",
    "TextType",
    "TextureType",
    "TiffEncodingType",
    "ToolType",
    "TransitionType",
    "TrimType",
    "TypeUnits",
    "UndefinedAreas",
    "UnderlineType",
    "Units",
    "Urgency",
    "Wartyle",
    "WaveType",
    "WhiteBalanceType",
    "ZigZagType",
]



# Import local modules
from photoshop.api._artlayer import ArtLayer
from photoshop.api._artlayers import ArtLayers
from photoshop.api._core import Photoshop
from photoshop.api._layers import Layers
from photoshop.api.enumerations import AnchorPosition
from photoshop.api.enumerations import BlendMode


class LayerSet(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def allLocked(self):
        return self.app.allLocked

    @allLocked.setter
    def allLocked(self, value):
        self.app.allLocked = value

    @property
    def artLayers(self):
        return ArtLayers(self.app.artLayers)

    @property
    def blendMode(self):
        return BlendMode(self.app.blendMode)

    @property
    def bounds(self):
        """The bounding rectangle of the layer set."""
        return self.app.bounds

    @property
    def enabledChannels(self):
        return self.app.enabledChannels

    @enabledChannels.setter
    def enabledChannels(self, value):
        self.app.enabledChannels = value

    @property
    def layers(self):
        return Layers(self.app.layers)

    @property
    def layerSets(self):
        # pylint: disable=import-outside-toplevel
        from ._layerSets import LayerSets

        return LayerSets(self.app.layerSets)

    @property
    def linkedLayers(self):
        """The layers linked to this layerSet object."""
        return self.app.linkedLayers or []

    @property
    def name(self) -> str:
        return self.app.name

    @name.setter
    def name(self, value):
        """The name of this layer set."""
        self.app.name = value

    @property
    def opacity(self):
        """The master opacity of the set."""
        return round(self.app.opacity)

    @opacity.setter
    def opacity(self, value):
        self.app.opacity = value

    @property
    def parent(self):
        return self.app.parent

    @property
    def visible(self):
        return self.app.visible

    @visible.setter
    def visible(self, value):
        self.app.visible = value

    def duplicate(self, relativeObject=None, insertionLocation=None):
        return LayerSet(self.app.duplicate(relativeObject, insertionLocation))

    def link(self, with_layer):
        self.app.link(with_layer)

    def add(self):
        self.app.add()

    def merge(self):
        return ArtLayer(self.app.merge())

    def move(self, relativeObject, insertionLocation):
        self.app.move(relativeObject, insertionLocation)

    def remove(self):
        layer = f'app.activeDocument.layerSets.getByName("{self.app.name}")'
        self.eval_javascript(f"{layer}.remove()")

    def resize(self, horizontal=None, vertical=None, anchor: AnchorPosition = None):
        self.app.resize(horizontal, vertical, anchor)

    def rotate(self, angle, anchor=None):
        self.app.rotate(angle, anchor)

    def translate(self, delta_x, delta_y):
        """Moves the position relative to its current position."""
        self.app.translate(delta_x, delta_y)

    def unlink(self):
        """Unlinks the layer set."""
        self.app.unlink()

    def __iter__(self):
        for layer in self.app:
            yield layer



# Import local modules
from photoshop.api._core import Photoshop


class RGBColor(Photoshop):
    object_name = "RGBColor"

    def __init__(self, parent):
        super().__init__(parent=parent)
        self.blue = self.app.blue
        self.green = self.app.green
        self.red = self.app.red

    @property
    def blue(self) -> int:
        return round(self.app.blue)

    @blue.setter
    def blue(self, value: int):
        self.app.blue = value

    @property
    def green(self) -> int:
        return round(self.app.green)

    @green.setter
    def green(self, value: int):
        self.app.green = value

    @property
    def red(self) -> int:
        return round(self.app.red)

    @red.setter
    def red(self, value: int):
        self.app.red = value

    @property
    def hexValue(self):
        return self.app.hexValue

    @hexValue.setter
    def hexValue(self, value):
        self.app.hexValue = value

    def __str__(self):
        return f"[red: {self.red}, green:{self.green},  blue:{self.blue})]"



# Import local modules
from photoshop.api._core import Photoshop


# pylint: disable=too-many-public-methods
class Channel(Photoshop):
    def __init__(self, parent):
        super().__init__(parent=parent)

    @property
    def color(self):
        return self.app.color

    @color.setter
    def color(self, value):
        self.app.color = value

    @property
    def histogram(self):
        return self.app.histogram

    @histogram.setter
    def histogram(self, value):
        self.app.histogram = value

    @property
    def kind(self):
        return self.app.kind

    @kind.setter
    def kind(self, value):
        self.app.kind = value

    @property
    def opacity(self):
        return self.app.opacity

    @opacity.setter
    def opacity(self, value):
        self.app.opacity = value

    @property
    def visible(self):
        return self.app.visible

    @visible.setter
    def visible(self, value):
        self.app.visible = value

    @property
    def name(self):
        return self.app.name

    def duplicate(self, targetDocument=None):
        self.app.duplicate(targetDocument)

    def merge(self):
        self.app.merge()

    def remove(self):
        channel = f'app.activeDocument.channels.getByName("{self.name}")'
        self.eval_javascript(f"{channel}.remove()")
